 /* Copyright 2023-2024 NVIDIA CORPORATION AND AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPI_PLUS_ARITHMETIC_AND_LOGICAL_OPERATIONS_H
#define NV_NPPI_PLUS_ARITHMETIC_AND_LOGICAL_OPERATIONS_H
 
/**
 * \file nppiPlus_arithmetic_and_logical_operations.h
 * Image Arithmetic and Logical Operations.
 */
 
/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 * \page image_arithmetic_and_logical_operations Arithmetic and Logical Operations
 * @defgroup image_arithmetic_and_logical_operations Arithmetic and Logical Operations
 * @ingroup nppi 
 * @{
 *
 * These functions can be found in the nppial library. Linking to only the sub-libraries that you use can significantly
 * save link time, application load time, and CUDA runtime startup time when using dynamic libraries.
 */

/** 
 * \section image_arithmetic_operations Arithmetic Operations
 * @defgroup image_arithmetic_operations Arithmetic Operations
 * The set of image processing arithmetic operations available in the library.
 * @{
 */

/** 
 * \section image_addc AddC
 * @defgroup image_addc AddC
 *
 * Adds a constant value to each pixel of an image. 
 *  
 * Note: If you use one of the device constant versions of these functions and the function called immediately preceeding that 
 * function generates that device constant you MUST either call cudaStreamSynchronize() or cudaDeviceSynchronize() before calling 
 * the device constant function. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image add constant, scale, then clamp to saturated value. 
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus
nppiAddC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                 NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image add constant, then clamp to saturated value. 
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus
nppiAddC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
             const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image add constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiAddC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                  const void * pConstant, void * pSrcDst, int nSrcDstStep,
                  NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image add constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiAddC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
               const void * pConstant, void * pSrcDst, int nSrcDstStep, 
               NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image add constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiAddDeviceC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                       const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                       NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image add constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiAddDeviceC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                   const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image add constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiAddDeviceC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                        const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                        NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image add constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiAddDeviceC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                     const void * pConstant, void * pSrcDst, int nSrcDstStep,  
                     NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_addc */ 

/** 
 * \section image_mulc MulC
 * @defgroup image_mulc MulC
 *
 * Multiplies each pixel of an image by a constant value. 
 *  
 * Note: If you use one of the device constant versions of these functions and the function called immediately preceeding that 
 * function generates that device constant you MUST either call cudaStreamSynchronize() or cudaDeviceSynchronize() before calling 
 * the device constant function. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image multiply constant, scale, then clamp to saturated value. 
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                 NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image multiply constant, then clamp to saturated value. 
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulC_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels, 
             const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                  const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                  NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
               const void * pConstant, void * pSrcDst, int nSrcDstStep, 
               NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image multiply constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulDeviceC_Sfs_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels, 
                       const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                       NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulDeviceC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                        const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                        NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image multiply constant, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulDeviceC_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels, 
                   const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply constant, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMulDeviceC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                     const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                     NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_mulc */ 

/** 
 * \section image_mulcscale MulCScale
 * @defgroup image_mulcscale MulCScale
 *
 * Multiplies each pixel of an image by a constant value then scales the result
 * by the maximum value for the data bit width. 
 *  
 * Note: If you use one of the device constant versions of these functions and the function called immediately preceeding that 
 * function generates that device constant you MUST either call cudaStreamSynchronize() or cudaDeviceSynchronize() before calling 
 * the device constant function. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image multiply by constant and scale by max bit width value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMulCScale_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                  const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply by constant and scale by max bit width value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMulCScale_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                    const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                    NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image multiply by constant and scale by max bit width value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMulDeviceCScale_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                        const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                        NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply by constant and scale by max bit width value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMulDeviceCScale_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                          const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                          NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_mulcscale */ 

/** 
 * \section image_subc SubC
 *  @defgroup image_subc SubC
 * Subtracts a constant value from each pixel of an image. 
 *  
 * Note: If you use one of the device constant versions of these functions and the function called immediately preceeding that 
 * function generates that device constant you MUST either call cudaStreamSynchronize() or cudaDeviceSynchronize() before calling 
 * the device constant function. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image subtract constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                 NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image subtract constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcDstChannels source \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
             const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image subtract constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                  const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                  NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image subtract constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
               const void * pConstant, void * pSrcDst, int nSrcDstStep, 
               NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image subtract constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubDeviceC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                       const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                       NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image subtract constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubDeviceC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                   const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image subtract constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubDeviceC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                        const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                        NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image subtract constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSubDeviceC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                     const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                     NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_subc */ 

/** 
 * \section image_divc DivC
 * @defgroup image_divc DivC
 *
 * Divides each pixel of an image by a constant value. 
 *  
 * Note: If you use one of the device constant versions of these functions and the function called immediately preceeding that 
 * function generates that device constant you MUST either call cudaStreamSynchronize() or cudaDeviceSynchronize() before calling 
 * the device constant function. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image divide by constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                 NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image divide by constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
             const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image divide by constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                  const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                  NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image divide by constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
               const void * pConstant, void * pSrcDst, int nSrcDstStep, 
               NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image divide by constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivDeviceC_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                       const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                       NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image divide by constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivDeviceC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                   const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image divide by constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivDeviceC_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                        const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                        NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image divide by constant then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDivDeviceC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                     const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                     NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_divc */ 

/** 
 * \section image_absdiffc AbsDiffC
 * @defgroup image_absdiffc AbsDiffC
 *
 * Determines absolute difference between each pixel of an image and a constant value. 
 *  
 * Note: If you use one of the device constant versions of these functions and the function called immediately preceeding that 
 * function generates that device constant you MUST either call cudaStreamSynchronize() or cudaDeviceSynchronize() before calling 
 * the device constant function. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image absolute difference with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                 NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image absolute difference with constant.
 * Supported data types include NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffC_Float_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                       const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                       NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image absolute difference with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                   const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place floating point image absolute difference with constant.
 * Supported data types include NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffC_I_Float_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                         const void * pConstant, void * pSrcDst, int nSrcDstStep, 
                         NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image absolute difference with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffDeviceC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                       const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                       NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Floating point image absolute difference with constant.
 * Supported data types include NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffDeviceC_Float_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                             const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
                             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image absolute difference with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffDeviceC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                         const void * pConstant, void * pSrcDst, int nSrcDstStep,
                         NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place floating point image absolute difference with constant.
 * Supported data types include NPP_8F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant device memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiffDeviceC_I_Float_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                               const void * pConstant, void * pSrcDst, int nSrcDstStep,
                               NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_absdiffc */ 

/** 
 * \section image_add Add
 * @defgroup image_add Add
 *
 * Pixel by pixel addition of two images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image addition, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAdd_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, void * pDst,  int nDstStep,  
                NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image addition then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAdd_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, void * pDst,  int nDstStep,  
            NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image addition, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAdd_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
                 NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image addition then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAdd_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
              const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
              NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_add */ 

/** 
 * \section image_addsquare AddSquare
 * @defgroup image_addsquare AddSquare
 *
 * Pixel by pixel addition of squared pixels from source image to floating point
 * pixel values of destination image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddSquare_IM_Ctx(NppDataType eSrcType, const void * pSrc, int nSrcStep, const Npp8u * pMask, int nMaskStep, 
                     Npp32f * pSrcDst, int nSrcDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image squared then added to in place floating point destination image.
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddSquare_I_Ctx(NppDataType eSrcType, const void * pSrc, int nSrcStep, 
                    Npp32f * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_addsquare */ 

/** 
 * \section image_addproduct AddProduct
 * @defgroup image_addproduct AddProduct
 * Pixel by pixel addition of product of pixels from two source images to
 * floating point pixel values of destination image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image product added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddProduct_IM_Ctx(NppDataType eSrcType, const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,
                      const Npp8u * pMask, int nMaskStep, Npp32f * pSrcDst, int nSrcDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image product added to in place floating point destination image.
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddProduct_I_Ctx(NppDataType eSrcType, const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,
                     Npp32f * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * One 16-bit floating point channel image product added to in place floating point destination image.
 * Supported data types include NPP_16F. 
 * Supported channel counts include NPP_CH_1. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddProduct16f_I_Ctx(const Npp16f * pSrc1,    int nSrc1Step,   const Npp16f * pSrc2, int nSrc2Step,
                              Npp16f * pSrcDst,  int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_addproduct */ 

/** 
 * \section image_addweighted AddWeighted
 * @defgroup image_addweighted AddWeighted
 * Pixel by pixel addition of alpha weighted pixel values from a source image to
 * floating point pixel values of destination image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * One alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddWeighted_IM_Ctx(NppDataType eSrcType, const void * pSrc, int nSrcStep, const Npp8u * pMask, int nMaskStep, 
                       Npp32f * pSrcDst, int nSrcDstStep,  NppiSize oSizeROI, Npp32f nAlpha, NppStreamContext nppStreamCtx);

/** 
 * One alpha weighted image added to in place floating point destination image.
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAddWeighted_I_Ctx(NppDataType eSrcType, const void * pSrc, int nSrcStep, 
                      Npp32f * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, Npp32f nAlpha, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_addweighted */ 

/** 
 * \section image_mul Mul
 * @defgroup image_mul Mul
 *
 * Pixel by pixel multiply of two images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image multiplication, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMul_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
                void * pDst,  int nDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image multiplication then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMul_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
            const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
            void * pDst,  int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image multiplication, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMul_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, const void * pSrc, int nSrcStep, 
                 void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image multiplication then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMul_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, const void * pSrc, int nSrcStep, 
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_mul */ 

/** 
 * \section image_mulscale MulScale
 * @defgroup image_mulscale MulScale
 *
 * Pixel by pixel multiplies each pixel of two images then scales the result by
 * the maximum value for the data bit width.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image multiplication then scale by maximum value for pixel bit width.
 * Supported data types include NPP_8U, NPP_16U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMulScale_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                 const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,
                 void * pDst,  int nDstStep,  
                 NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image multiply by constant and scale by max bit width value.
 * Supported data types include NPP_8U, NPP_16U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiMulScale_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                   const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_mulscale */ 

/** 
 * \section image_sub Sub
 * @defgroup image_sub Sub
 *
 * Pixel by pixel subtraction of two images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image subtraction, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSub_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, void * pDst, int nDstStep, 
                NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image subtraction then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSub_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
            const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, void * pDst, int nDstStep, 
            NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image subtraction, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSub_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, const void * pSrc, int nSrcStep, 
                 void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image subtraction then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSub_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, const void * pSrc, int nSrcStep, 
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_sub */ 

/** 
 * \section image_div Div
 * @defgroup image_div Div
 *
 * Pixel by pixel division of two images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image division, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDiv_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
                const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
                void * pDst, int nDstStep,  NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image division, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDiv_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                 const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
                 NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image division.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDiv_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
            void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image division.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref destination_image_pointer.
 * \param nSrcDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDiv_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
              NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_div */ 

/** 
 * \section image_divround Div_Round
 * @defgroup image_divround Div_Round
 *
 * Pixel by pixel division of two images using result rounding modes.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image division, scale by \f$2^(-nScaleFactor)\f$, then round and clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDiv_Round_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                      const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
                      void * pDst, int nDstStep, NppiSize oSizeROI, NppRoundMode rndMode, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image division, scale by \f$2^(-nScaleFactor)\f$, then round and clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiDiv_Round_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                       const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
                       NppiSize oSizeROI, NppRoundMode rndMode, int nScaleFactor, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_divround */ 

/** 
 * \section image_abs Abs
 * @defgroup image_abs Abs
 *
 * Absolute value of each pixel value in an image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image integer absolute value.
 * Supported data types include NPP_16S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc, int nSrcStep, void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image integer absolute value.
 * Supported data types include NPP_16S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbs_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image floating point absolute value.
 * Supported data types include NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsf_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
             const void * pSrc, int nSrcStep, void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image floating point absolute value.
 * Supported data types include NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsf_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
               void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_abs */

/** 
 * \section image_absdiff AbsDiff
 * @defgroup image_absdiff AbsDiff
 *
 * Pixel by pixel absolute difference between two images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Absolute difference of image1 minus image2.
 * Supported data types include NPP_8U, NPP_16U, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiff_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
                void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Absolute difference of floating point image1 minus image2.
 * Supported data types include NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAbsDiff_Float_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                      const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
                      void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_absdiff */ 

/** 
 * \section image_sqr Sqr
 * @defgroup image_sqr Sqr
 *
 * Square each pixel in an image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image squared, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqr_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                const void * pSrc, int nSrcStep, void * pDst, int nDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image squared, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqr_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                 void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image squared.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqr_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc, int nSrcStep, void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image squared.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqr_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              void * pSrcDst,  int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_sqr */ 

/** 
 * \section image_sqrt Sqrt
 * @defgroup image_sqrt Sqrt
 *
 * Pixel by pixel square root of each pixel in an image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image square root, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqrt_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                 const void * pSrc, int nSrcStep, void * pDst,  int nDstStep,  NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image square root, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqrt_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                  void * pSrcDst,  int nSrcDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image square root.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqrt_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
             const void * pSrc, int nSrcStep, void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image square root.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiSqrt_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                      void * pSrcDst,  int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_sqrt */ 

/** 
 * \section image_ln Ln
 * @defgroup image_ln Ln
 *
 * Pixel by pixel natural logarithm of each pixel in an image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image natural logarithm, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiLn_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
               const void * pSrc, int nSrcStep, void * pDst, int nDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image natural logarithm, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiLn_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image natural logarithm then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiLn_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
           const void * pSrc, int nSrcStep, void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image natural logarithm.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiLn_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
             void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_ln */ 

/** 
 * \section image_exp Exp
 * @defgroup image_exp Exp
 *
 * Exponential value of each pixel in an image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image exponential, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiExp_Sfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                const void * pSrc, int nSrcStep, void * pDst, int nDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place image exponential, scale by \f$2^(-nScaleFactor)\f$, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiExp_ISfs_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                 void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Image exponential.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiExp_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc, int nSrcStep, void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image exponential.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiExp_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_exp */ 

/** @} image_arithmetic_operations */ 

/** 
 * \section image_logical_operations Logical Operations
 * @defgroup image_logical_operations Logical Operations
 * The set of image processing logical operations available in the library.
 * @{
 */

/** 
 * \section image_andc AndC
 * @defgroup image_andc AndC
 *
 * Pixel by pixel logical AND of an image with a constant.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical AND with constant. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match source \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAndC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
             const void * pSrc, int nSrcStep, const void * pConstant, void * pDst,  int nDstStep,  
             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image AND with constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAndC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels, 
               const void * pConstant, void * pSrcDst, int nSrcDstStep, 
               NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_andc */ 

/** 
 * \section image_orc OrC
 * @defgroup image_orc Orc
 *
 * Pixel by pixel logical OR of an image with a constant.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical OR with constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiOrC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc1, int nSrc1Step, const void * pConstant, 
                         void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image logical OR with constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiOrC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              const void * pConstant, void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_orc */ 

/** 
 * \section image_xorc XorC
 * @defgroup image_xorc xOrC
 *
 * Pixel by pixel logical exclusive OR of an image with a constant.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical exclusive OR with constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiXorC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
             const void * pSrc1, int nSrc1Step, const void * pConstant, 
             void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image logical exclusive OR with constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant host memory Constant pointer (must match \ref image_data_type and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiXorC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
               const void * pConstant, void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_xorc */ 

/** 
 * \section image_rshiftc RShiftC
 * @defgroup image_rshiftc RShiftC
 *
 * Pixel by pixel right shift of an image by a constant value.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image right shift by unsigned constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pConstant unsigned host memory Constant pointer (must match image data size and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiRShiftC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                const void * pSrc1, int nSrc1Step, const Npp32u * pConstant, 
                void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image right shift by unsigned constant.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant unsigned host memory Constant pointer (must match image data size and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiRShiftC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                  const Npp32u * pConstant, void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_rshiftc */ 

/** 
 * \section image_lshiftc LShiftC
 * @defgroup image_lshiftc LShiftC
 *
 * Pixel by pixel left shift of an image by a constant value.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image left shift by unsigned constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pConstant unsigned host memory Constant pointer (must match image data size and number of channels). 
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiLShiftC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                const void * pSrc1, int nSrc1Step, const Npp32u * pConstant, 
                void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image left shift by unsigned constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pConstant unsigned host memory Constant pointer (must match image data size and number of channels). 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiLShiftC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                  const Npp32u * pConstant, void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_lshiftc */ 

/** 
 * \section image_and And
 * @defgroup image_and And
 *
 * Pixel by pixel logical and of images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical AND.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAnd_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
            void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image logical AND.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAnd_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              const void * pSrc, int nSrcStep, 
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_and */ 

/** 
 * \section image_or Or
 * @defgroup image_or Or
 *
 * Pixel by pixel logical OR of images.
 *
 * @{
 */
 
#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical OR.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S.  
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiOr_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
           const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
           void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image logical OR.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiOr_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
             const void * pSrc, int nSrcStep, 
             void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_or */ 

/** 
 * \section image_xor Xor
 * @defgroup image_xor Xor
 *
 * Pixel by pixel logical exclusive OR of images.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical exclusive OR.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiXor_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
            void * pDst,  int nDstStep,  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image logical exclusive OR.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiXor_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              const void * pSrc, int nSrcStep, 
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_xor */ 

/** 
 * \section image_not Not
 * @defgroup image_not Not
 *
 * Pixel by pixel logical NOT of image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image logical NOT.
 * Supported data types include NPP_8U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiNot_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pSrc, int nSrcStep, void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image logical NOT.
 * Supported data types include NPP_8U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiNot_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_not */ 

/** @} image_logical_operations */ 

/** 
 * \section image_alpha_composition_operations Alpha Composition
 * @defgroup image_alpha_composition_operations Alpha Composition
 * The set of alpha composition operations available in the library.
 * @{
 */

/** 
 * \section image_alphacompc AlphaCompC
 * @defgroup image_alphacompc AlphaCompC
 *
 * Composite two images using constant alpha values.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image composition using constant alpha.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pAlpha1 Image alpha opacity (0 - max channel pixel value).
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pAlpha2 Image alpha opacity (0 - max channel pixel value).
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param eAlphaOp alpha-blending operation..
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAlphaCompC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                   const void * pSrc1, int nSrc1Step, const void * pAlpha1, const void * pSrc2, int nSrc2Step, const void * pAlpha2,
                   void * pDst, int nDstStep, NppiSize oSizeROI, NppiAlphaOp eAlphaOp, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_alphacompc */ 

/** 
 * @defgroup image_alphapremulc AlphaPremulC
 * 
 * Premultiplies pixels of an image using a constant alpha value.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image premultiplication using constant alpha.
 * Supported data types include NPP_8U, NPP_16U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pAlpha1 Image alpha opacity (0 - max channel pixel value).
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAlphaPremulC_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                     const void * pSrc1, int nSrc1Step, const void * pAlpha1, void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * In place image premultiplication using constant alpha.
 * Supported data types include NPP_8U, NPP_16U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pAlpha1 Image alpha opacity (0 - max channel pixel value).
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAlphaPremulC_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                       const void * pAlpha1, void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_alphapremulc */ 

/** 
 * \section image_alphacomp AlphaComp
 * @defgroup image_alphacomp AlphaComp
 *
 * Composite two images using alpha opacity values contained in each image.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image composition using image alpha values (0 - max channel pixel value).
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param eAlphaOp alpha-blending operation..
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAlphaComp_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                  const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step, 
                  void * pDst,  int nDstStep,  NppiSize oSizeROI, NppiAlphaOp eAlphaOp, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_alphacomp */ 

/** 
 * @defgroup image_alphapremul AlphaPremul
 * 
 * Premultiplies image pixels by image alpha opacity values.
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Four channel premultiplication with pixel alpha (0 - max channel pixel value).
 * Supported data types include NPP_8U, NPP_16U. 
 * Supported channel counts include NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAlphaPremul_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                    const void * pSrc1, int nSrc1Step, void * pDst, int nDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Four channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
 * Supported data types include NPP_8U, NPP_16U. 
 * Supported channel counts include NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */

NPP_EXTERN_C
NppStatus 
nppiAlphaPremul_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                      void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} image_alphapremul */ 

/** @} image_alpha_composition*/ 

/** @} image_arithmetic_and_logical_operations */
 
#endif /* NV_NPPI_PLUS_ARITHMETIC_AND_LOGICAL_OPERATIONS_H */
