 /* Copyright 2009-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited.
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPI_PLUS_DATA_EXCHANGE_AND_INITIALIZATION_H
#define NV_NPPI_PLUS_DATA_EXCHANGE_AND_INITIALIZATION_H
 
/**
 * \file nppiPlus_data_exchange_and_initialization.h
 * NPP Image Processing Functionality.
 */
 
/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 *  \page image_data_exchange_and_initialization Data Exchange and Initialization
 *  @defgroup image_data_exchange_and_initialization Data Exchange and Initialization
 *  @ingroup nppi
 *
 * Functions for initializing, copying and converting image data.
 *
 * @{
 *
 * These functions can be found in the nppidei library. Linking to only the sub-libraries that you use can significantly
 * save link time, application load time, and CUDA runtime startup time when using dynamic libraries.
 *
 */

/** 
 * \section image_set_operations Image Set Operations 
 * @defgroup image_set_operations Image Set Operations
 * Functions for setting all pixels within the ROI to a specific value.
 *
 * @{
 *
 */

/** 
 * \section image_set Set
 * @defgroup image_set Set
 * Functions for setting all pixels within the ROI to a specific value.
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * \subsection CommonImageSetParameters Common parameters for nppiSet functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pValues Host memory pointer to the pixel color value(s) to be used to set.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */


#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Set image pixel color value(s).
 * 
 * For common parameter descriptions, see \ref CommonImageSetParameters. Common parameters for nppiSet functions</a>.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSet_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
            const void * pValues, void * pDst, int nDstStep, 
            NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_set */

/** 
 * \section image_masked_set Masked Set
 * @defgroup image_masked_set Masked Set
 * The masked set primitives have an additional "mask image" input. The mask  
 * controls which pixels within the ROI are set. For details see \ref masked_operation.
 *
 * \subsection CommonImageMaskedSetParameters Common parameters for nppiSet_CXM functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pValues Host memory pointer to the pixel color value(s) to be used to set.
 * \param pDst Pointer \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Set image pixel color value(s) using mask control. 
 * For A4 channel counts destination alpha channel remains unaffected.
 * 
 * For common parameter descriptions, see \ref CommonImageMaskedSetParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSet_M_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
              const void * pValues, void * pDst, int nDstStep, 
              NppiSize oSizeROI, const Npp8u * pMask, int nMaskStep, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_masked_set */

/** 
 * \section image_channel_set Channel Set
 * @defgroup image_channel_set Channel Set
 * The selected-channel set primitives set a single color channel in multi-channel images
 * to a given value. The channel is selected by adjusting the pDst pointer to point to 
 * the desired color channel (see \ref channel_of_interest).
 *
 * \subsection CommonImageChannelSetParameters Common parameters for nppiSet_CXC functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eDstChannels destination \ref image_channels. 
 * \param pValue Host memory pointer to the pixel color value to be used to set.
 * \param pDst \ref select_destination_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image pixel color value set affecting only single channel.
 * For A4 channel counts destination alpha channel remains unaffected.
 * 
 * For common parameter descriptions, see \ref CommonImageChannelSetParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSetChannel_Ctx(NppDataType eSrcDstType, NppiChannels eDstChannels,
                   const void * pValue, void * pDst, int nDstStep, 
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_channel_set */

/** @} image_set_operations */

/** 
 * \section image_copy_operations Image Copy Operations
 * @defgroup image_copy_operations Image Copy Operations
 * Functions for copying image pixels.
 *
 * @{
 *
 */

/** 
 * \section image_copy Copy
 * @defgroup image_copy Copy
 * Copy pixels from one image to another.
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * \subsection CommonImageCopyParameters Common parameters for nppiCopy functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image copy.
 * 
 * For common parameter descriptions, see \ref CommonImageCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopy_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
             const void * pSrc, int nSrcStep, void * pDst, int nDstStep, 
             NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_copy */

/** 
 * \section image_masked_copy Masked Copy
 * @defgroup image_masked_copy Masked Copy
 * The masked copy primitives have an additional "mask image" input. The mask  
 * controls which pixels within the ROI are copied. For details see \ref masked_operation.
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * \subsection CommonImageMaskedCopyParameters Common parameters for nppiCopy_CXM functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image copy with mask control.
 * 
 * For common parameter descriptions, see \ref CommonImageMaskedCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopy_M_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
               const void * pSrc, int nSrcStep, void * pDst, int nDstStep, 
               NppiSize oSizeROI, const Npp8u * pMask, int nMaskStep, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_masked_copy */


/** 
 * \section image_channel_copy Channel Copy
 * @defgroup image_channel_copy Copy Channel
 * The channel copy primitives copy a single color channel from a multi-channel source image
 * to any other color channel in a multi-channel destination image. The channel is selected 
 * by adjusting the respective image pointers to point to the desired color channel 
 * (see \ref channel_of_interest).
 *
 * \subsection CommonImageChannelCopyParameters Common parameters for nppiCopy_CXC functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref select_source_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref select_destination_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image copy selected channel. 
 * 
 * For common parameter descriptions, see \ref CommonImageChannelCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopyChannel_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                    const void * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                    NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_channel_copy */

/** 
 * \section image_extract_channel_copy Extract Channel Copy
 * @defgroup image_extract_channel_copy Extract Channel Copy
 * The channel extract primitives copy a single color channel from a multi-channel source image
 * to singl-channel destination image. The channel is selected by adjusting the source image pointer
 * to point to the desired color channel (see \ref channel_of_interest).
 *
 * \subsection CommonImageExtractChannelCopyParameters Common parameters for nppiCopyExtractChannel_CXC1 functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc \ref select_source_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image extract source channel copy to single destination channel.
 * 
 * For common parameter descriptions, see \ref CommonImageExtractChannelCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopyExtractChannel_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
                           const void * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                           NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_extract_channel_copy */

/** 
 * \section image_insert_channel_copy Insert Channel Copy
 * @defgroup image_insert_channel_copy Insert Channel Copy
 * The channel insert primitives copy a single-channel source image into one of the color channels
 * in a multi-channel destination image. The channel is selected by adjusting the destination image pointer
 * to point to the desired color channel (see \ref channel_of_interest).
 *
 * \subsection CommonImageInsertChannelCopyParameters Common parameters for nppiCopyInsertChannel_C1CX functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref select_destination_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image single source channel copy insert (replace) destination channel.
 * 
 * For common parameter descriptions, see \ref CommonImageInsertChannelCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopyInsertChannel_Ctx(NppDataType eSrcDstType, NppiChannels eDstChannels,
                          const void * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                          NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_insert_channel_copy */


/** 
 * \section image_packed_to_planar_channel_copy Packed To Planar Channel Copy
 * @defgroup image_packed_to_planar_channel_copy Packed To Planar Channel Copy
 * Split a packed multi-channel image into multiple single channel planes.
 *
 * E.g. copy the three channels of an RGB image into three separate single-channel
 * images.
 *
 * \subsection CommonImagePackedToPlanarChannelCopyParameters Common parameters for nppiCopy_CXPX functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported source channel counts include NPP_CH_3, NPP_CH_4. 
 * Supported destination channel counts include NPP_CH_P3, NPP_CH_P4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcChannels source \ref image_channels. 
 * \param eDstChannels destination \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param aDst \ref destination_planar_image_pointer_array.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Packed to planar same channel count image copy.
 * 
 * For common parameter descriptions, see \ref CommonImagePackedToPlanarChannelCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus
nppiCopy_CXPX_Ctx(NppDataType eSrcDstType, NppiChannels eDstChannels,
                  const void * pSrc, int nSrcStep, void * aDst[4], int nDstStep, 
                  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_packed_to_planar_channel_copy */


/** 
 * \section image_planar_to_packed_channel_copy Planar To Packed Channel Copy
 * @defgroup image_planar_to_packed_channel_copy Planar To Packed Channel Copy
 * Combine multiple image planes into a packed multi-channel image.
 *
 * E.g. copy three single-channel images into a single 3-channel image.
 *
 * \subsection CommonImagePlanarToPackedChannelCopyParameters Common parameters for nppiCopy_PXCX functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported source channel counts include NPP_CH_P3, NPP_CH_P4. 
 * Supported destination channel counts include NPP_CH_C3, NPP_CH_C4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcChannels source \ref image_channels. 
 * \param eDstChannels destination \ref image_channels. 
 * \param aSrc Planar \ref source_image_pointer.
 * \param nSrcStep \ref source_planar_image_pointer_array.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Planar to packed same channel count image copy.
 * 
 * For common parameter descriptions, see \ref CommonImagePlanarToPackedChannelCopyParameters.
 *
 */
NPP_EXTERN_C
NppStatus
nppiCopy_PXCX_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
                  void * const aSrc[4], int nSrcStep, void * pDst, int nDstStep, 
                  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_planar_to_packed_channel_copy */

/** 
 * \section image_copy_constant_border Copy Constant Border
 * @defgroup image_copy_constant_border Copy Constant Border
 * Methods for copying images and padding borders with a constant, user-specifiable color.
 * For A4 channel counts destination alpha channel remains unaffected.
 * 
 * \subsection CommonImageCopyConstantBorderParameters Common parameters for nppiCopyConstBorder functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSrcSizeROI Size of the source region of pixels.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * \param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the constant border color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * \param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * \param pBorderValues Host memory pointer to the single color value or RGBA color values of set as border pixels.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image copy with constant border color.
 * 
 * For common parameter descriptions, see \ref CommonImageCopyConstantBorderParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopyConstBorder_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                        const void * pSrc, int nSrcStep, NppiSize oSrcSizeROI,
                        void * pDst, int nDstStep, NppiSize oDstSizeROI,
                        int nTopBorderHeight, int nLeftBorderWidth,
                        void * pBorderValues, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_copy_constant_border */

/** 
 * \section image_copy_replicate_border Copy Replicate Border
 * @defgroup image_copy_replicate_border Copy Replicate Border
 * Methods for copying images and padding borders with a replicates of the nearest source image pixel color.
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * \subsection CommonImageCopyReplicateBorderParameters Common parameters for nppiCopyReplicateBorder functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSrcSizeROI Size of the source region of pixels.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and nearest source image pixel color (outer part).
 * \param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the nearest source image pixel color.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * \param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image copy with nearest source image pixel border color replication.
 * 
 * For common parameter descriptions, see \ref CommonImageCopyReplicateBorderParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopyReplicateBorder_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                            const void * pSrc, int nSrcStep, NppiSize oSrcSizeROI,
                            void * pDst, int nDstStep, NppiSize oDstSizeROI,
                            int nTopBorderHeight, int nLeftBorderWidth,
                            NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_copy_replicate_border */

/** 
 * \section image_copy_wrap_border Copy Wrap Border
 * @defgroup image_copy_wrap_border Copy Wrap Border
 * Methods for copying images and padding borders with wrapped replications of the source image pixel colors.
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * \subsection CommonImageCopyWrapBorderParameters Common parameters for nppiCopyWrapBorder functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_16F, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSrcSizeROI Size of the source region of pixels.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and a border consisting of wrapped replication of the source image pixel colors (outer part).
 * \param nTopBorderHeight Height (in pixels) of the top border. The number of pixel rows at the top of the
 *      destination ROI that will be filled with the wrapped replication of the corresponding column of source image pixels colors.
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * \param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image copy with borders wrapped by replication of source image pixel colors.
 * 
 * For common parameter descriptions, see \ref CommonImageCopyWrapBorderParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopyWrapBorder_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                       const void * pSrc, int nSrcStep, NppiSize oSrcSizeROI,
                       void * pDst, int nDstStep, NppiSize oDstSizeROI,
                       int nTopBorderHeight, int nLeftBorderWidth,
                       NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_copy_wrap_border */

/** 
 * \section image_copy_sub_pixel Copy Sub-Pixel
 * @defgroup image_copy_sub_pixel Copy Sub-Pixel
 * Functions for copying linearly interpolated images using source image subpixel coordinates. 
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * \subsection CommonImageCopySubPixelParameters Common parameters for nppiCopySubpix functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * \param nDx Fractional part of source image X coordinate.
 * \param nDy Fractional part of source image Y coordinate.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 *  Linearly interpolated source image subpixel coordinate color copy.
 * 
 * For common parameter descriptions, see \ref CommonImageCopySubPixelParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCopySubpix_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                   const void * pSrc, int nSrcStep,
                   void * pDst, int nDstStep, NppiSize oDstSizeROI,
                   Npp32f nDx, Npp32f nDy,
                   NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_copy_sub_pixel */

/** @} image_copy_operations */

/** 
 * \section image_convert Convert Bit Depth
 * @defgroup image_convert Convert Bit Depth
 * Functions for converting bit depth without scaling.
 *
 * @{
 *
 */

/** 
 * \section image_convert_increase Convert To Increased Bit Depth
 * @defgroup image_convert_increase Convert To Increased Bit Depth
 * The integer conversion methods do not involve any scaling. Also, even when increasing the bit-depth
 * loss of information may occur:
 * - When converting integers (e.g. Npp32u) to float (e.g. Npp32f) integer value not accurately representable 
 *   by the float are rounded to the closest floating-point value.
 * - When converting signed integers to unsigned integers all negative values are lost (saturated to 0).
 * For A4 channel counts destination alpha channel remains unaffected.
 *  
 * Note that all pointers and step sizes for images with 16f (Npp16f) data types perform best when they are at least 16 byte aligned. 
 *  
 * \subsection CommonImageConvertToIncreasedBitDepthParameters Common parameters for nppiConvert to increased bit depth functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image increase bit depth conversion.
 * 
 * For common parameter descriptions, see \ref CommonImageConvertToIncreasedBitDepthParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiConvert_Up_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                   const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_convert_increase */

/** 
 * \section image_convert_decrease Convert To Decreased Bit Depth
 * @defgroup image_convert_decrease Convert To Decreased Bit Depth
 * The integer conversion methods do not involve any scaling. When converting floating-point values
 * to integers the user may choose the most appropriate rounding-mode. Typically information is lost when
 * converting to lower bit depth:
 * - All converted values are saturated to the destination type's range. E.g. any values larger than
 *   the largest value of the destination type are clamped to the destination's maximum.
 * - Converting floating-point values to integer also involves rounding, effectively loosing all
 *   fractional value information in the process.
 * For A4 channel counts destination alpha channel remains unaffected.
 *  
 * Note that all pointers and step sizes for images with 16f (Npp16f) data types perform best when they are at least 16 byte aligned. 
 *
 * \subsection CommonImageConvertToDecreasedBitDepthParameters Common parameters for nppiConvert to decreased bit depth functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param eRoundMode \ref rounding_mode_parameter.
 * \param nScaleFactor \ref integer_result_scaling, 0 for no scaling.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image decrease bit depth conversion.
 * 
 * For common parameter descriptions, see \ref CommonImageConvertToDecreasedBitDepthParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiConvert_Down_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                     const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                     NppiSize oSizeROI, NppStreamContext nppStreamCtx);

/** 
 * Image decrease bit depth conversion with scaling.
 * 
 * For common parameter descriptions, see \ref CommonImageConvertToDecreasedBitDepthParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiConvert_Down_Sfs_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                         const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                         NppRoundMode eRoundMode, int nScaleFactor,
                         NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_convert_decrease */

/** @} image_convert */

/** 
 * \section image_scale Scale Bit Depth
 * @defgroup image_scale Scale Bit Depth
 * Functions for scaling bit depth up or down.
 *
 * @{
 *
 */

/** 
 * \section image_scale_to_higher_bit_depth Scale To Higher Bit Depth
 * @defgroup image_scale_to_higher_bit_depth Scale To Higher Bit Depth
 * Functions for scaling images to higher bit depth.
 *
 * To map source pixel srcPixelValue to destination pixel dstPixelValue the following equation is used:
 * 
 *      dstPixelValue = dstMinRangeValue + scaleFactor * (srcPixelValue - srcMinRangeValue)
 *
 * where scaleFactor = (dstMaxRangeValue - dstMinRangeValue) / (srcMaxRangeValue - srcMinRangeValue).
 *
 * For conversions between integer data types, the entire integer numeric range of the input data type is mapped onto 
 * the entire integer numeric range of the output data type.
 * For A4 channel counts destination alpha channel remains unaffected.
 *
 * For conversions to floating point data types the floating point data range is defined by the user supplied floating point values
 * of nMax and nMin which are used as the dstMaxRangeValue and dstMinRangeValue respectively in the scaleFactor and dstPixelValue 
 * calculations and also as the saturation values to which output data is clamped.
 *
 * When converting from floating-point values to integer values, nMax and nMin are used as the srcMaxRangeValue and srcMinRangeValue
 * respectively in the scaleFactor and dstPixelValue calculations. Output values are saturated and clamped to the full output integer
 * pixel value range.
 *
 * \subsection CommonImageScaleToHigherBitDepthParameters Common parameters for nppiScale to higher bit depth functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nMin specifies the minimum saturation value to which every output value will be clamped, ignored for integer destinations.
 * \param nMax specifies the maximum saturation value to which every output value will be clamped, ignored for integer destinations.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image scale bit depth up conversion.
 *
 * For common parameter descriptions, see \ref CommonImageScaleToHigherBitDepthParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiScale_Up_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                 const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                 NppiSize oSizeROI, Npp32f nMin, Npp32f nMax, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_scale_to_higher_bit_depth */

/** 
 * \section image_scale_to_lower_bit_depth Scale To Lower Bit Depth
 * @defgroup image_scale_to_lower_bit_depth Scale To Lower Bit Depth
 * Functions for scaling images to lower bit depth.
 *
 * To map source pixel srcPixelValue to destination pixel dstPixelValue the following equation is used:
 * 
 *      dstPixelValue = dstMinRangeValue + scaleFactor * (srcPixelValue - srcMinRangeValue)
 *
 * where scaleFactor = (dstMaxRangeValue - dstMinRangeValue) / (srcMaxRangeValue - srcMinRangeValue).
 *
 * For conversions between integer data types, the entire integer numeric range of the input data type is mapped onto 
 * the entire integer numeric range of the output data type.
 *
 * For conversions to floating point data types the floating point data range is defined by the user supplied floating point values
 * of nMax and nMin which are used as the dstMaxRangeValue and dstMinRangeValue respectively in the scaleFactor and dstPixelValue 
 * calculations and also as the saturation values to which output data is clamped.
 *
 * When converting from floating-point values to integer values, nMax and nMin are used as the srcMaxRangeValue and srcMinRangeValue
 * respectively in the scaleFactor and dstPixelValue calculations. Output values are saturated and clamped to the full output integer
 * pixel value range.
 *
 * \subsection CommonImageScaleToLowerBitDepthParameters Common parameters for nppiScale to lower bit depth functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nMin specifies the minimum saturation value to which every output value will be clamped, ignored for integer source pixels.
 * \param nMax specifies the maximum saturation value to which every output value will be clamped, ignored for integer source pixels.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, NPP_SCALE_RANGE_ERROR indicates an error condition if nMax <= nMin.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image scale bit depth down conversion.
 * 
 * For common parameter descriptions, see \ref CommonImageScaleToLowerBitDepthParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiScale_Down_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                   const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                   NppiSize oSizeROI, Npp32f nMin, Npp32f nMax, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_scale_to_lower_bit_depth */

/** @} image_scale */

/** 
 * \section image_duplicate_channel Duplicate Channel
 * @defgroup image_duplicate_channel Duplicate Channel
 * Functions for duplicating a single channel image in a multiple channel image.
 *
 * \subsection CommonImageDuplicateChannelParameters Common parameters for nppiDup functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType destination \ref image_data_type.
 * \param eDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image, source image ROI is assumed to be same as destination image ROI.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Duplicate single channel source image in all destination channels. 
 *
 * For common parameter descriptions, see \ref CommonImageDuplicateChannelParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiDupChannel_Ctx(NppDataType eSrcDstType, NppiChannels eDstChannels,
                   const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                   NppiSize oDstSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_duplicate_channel */


/** 
 * \section image_transpose Transpose Image
 * @defgroup image_transpose Transpose Image
 * Functions for transposing images of various types. Like matrix transpose, image transpose is a mirror along the image's
 * diagonal (upper-left to lower-right corner).
 * 
 * \subsection CommonImageTransposeParameters Common parameters for nppiTranspose functions include:
 *
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst Pointer to the destination ROI.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image transpose.
 *
 * For common parameter descriptions, see \ref CommonImageTransposeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiTranspose_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                  const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                  NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_transpose */

/** 
 * \section image_swap_channels Swap Channels
 * @defgroup image_swap_channels Swap Channels
 * Functions for swapping and duplicating channels in multiple channel images. 
 * The methods support arbitrary permutations of the original channels, including replication and
 * setting one or more channels to a constant value.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image swap channels to 3 channel destination.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcChannels source \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSwapChannels_C3R_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
                         const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                         NppiSize oSizeROI, const int aDstOrder[3], NppStreamContext nppStreamCtx);

/** 
 * In place image swap channels o 3 channel destination.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcChannels source \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [2,1,0] converts this to BGR
 *      channel order.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSwapChannels_C3IR_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
                          void * pSrcDst, int nSrcDstStep, 
                          NppiSize oSizeROI, const int aDstOrder[3], NppStreamContext nppStreamCtx);

/** 
 * Image swap channels to 4 channel destination.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcChannels source \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * \param pValue (V) Host memory pointer to single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      pValue[0] is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified. pValue is ignored if both eSrcChannels is NPP_CH_4.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSwapChannels_C4R_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
                         const void  * pSrc, int nSrcStep, void * pDst, int nDstStep, 
                         NppiSize oSizeROI, const int aDstOrder[4], const void * pValue, NppStreamContext nppStreamCtx);

/** 
 * In place image swap channels o 4 channel destination.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_3, NPP_CH_4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcChannels source \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref in_place_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param aDstOrder Host memory integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGB image, aDstOrder = [3,2,1,0] converts this to VBGR
 *      channel order.
 * \param pValue (V) Host memory pointer to single channel constant value that can be replicated in one or more of the 4 destination channels.
 *      pValue[0] is either written or not written to a particular channel depending on the aDstOrder entry for that destination
 *      channel. An aDstOrder value of 3 will output nValue to that channel, an aDstOrder value greater than 3 will leave that
 *      particular destination channel value unmodified. pValue is ignored if both eSrcChannels is NPP_CH_4.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSwapChannels_C4IR_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
                          void * pSrcDst, int nSrcDstStep, 
                          NppiSize oSizeROI, const int aDstOrder[4], const void * pValue, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} image_swap_channels */

/** @} image_data_exchange_and_initialization */

#endif /* NV_NPPI_PLUS_DATA_EXCHANGE_AND_INITIALIZATION_H */
