 /* Copyright 2009-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPI_PLUS_THRESHOLD_AND_COMPARE_OPERATIONS_H
#define NV_NPPI_PLUS_THRESHOLD_AND_COMPARE_OPERATIONS_H
 
/**
 * \file nppiPlus_threshold_and_compare_operations.h
 * NPP Image Processing Functionality.
 */

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 *  \page image_threshold_and_compare_operations Threshold and Compare Operations
 *  @defgroup image_threshold_and_compare_operations Threshold and Compare Operations
 *  @ingroup nppi
 *
 * Methods for pixel-wise threshold and compare operations.
 *
 * @{
 *
 * These functions can be found in the nppitc library. Linking to only the sub-libraries that you use can significantly
 * save link time, application load time, and CUDA runtime startup time when using dynamic libraries.
 *
 */

/** 
 * \section image_thresholding_operations Thresholding Operations
 * @defgroup image_thresholding_operations Thresholding Operations
 * Various types of image thresholding operations.
 *
 * @{
 *
 */

/** 
 * \section image_threshold_operations Threshold Operations
 * @defgroup image_threshold_operations Threshold Operations
 * Threshold image pixels.
 *
 * \section CommonThresholdParameters Common parameters for nppiThreshold non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold Host memory pointer to the threshold value(s) of the same data type.
 * \param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP pThreshold) is true, the pixel is set
 * to pThreshold, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                  const void * pSrc, int nSrcStep,
                  void * pDst, int nDstStep, NppiSize oSizeROI, 
                  const void * pThreshold, NppCmpOp eComparisonOperation, NppStreamContext nppStreamCtx); 

/** 
 * In place image threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP pThreshold) is true, the pixel is set
 * to pThreshold, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                    void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, 
                    const void * pThreshold, NppCmpOp eComparisonOperation, NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_operations */

/** 
 * \section image_threshold_greater_than_operations Threshold Greater Than Operations
 * @defgroup image_threshold_greater_than_operations Threshold Greater Than Operations
 * Threshold greater than image pixels.
 *
 * \section CommonThresholdGreaterThanParameters Common parameters for nppiThreshold_GT non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold The threshold value.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image greater than threshold.
 * If for a comparison operations sourcePixel is greater than pThreshold is true, the pixel is set
 * to pThreshold, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdGreaterThanParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_GT_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                     const void * pSrc, int nSrcStep,
                     void * pDst, int nDstStep, NppiSize oSizeROI, 
                     const void * pThreshold, NppStreamContext nppStreamCtx); 

/** 
 * In place image greater than threshold.
 * If for a comparison operations sourcePixel is greater than pThreshold is true, the pixel is set
 * to pThreshold, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdGreaterThanParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_GT_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                       void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, 
                       const void * pThreshold, NppStreamContext nppStreamCtx); 


#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_greater_than_operations */

/** 
 * \section image_threshold_less_than_operations Threshold Less Than Operations
 * @defgroup image_threshold_less_than_operations Threshold Less Than Operations
 * Threshold less than image pixels.
 *
 * \section CommonThresholdLessThanParameters Common parameters for nppiThreshold_LT non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold The threshold value.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image less than threshold.
 * If for a comparison operations sourcePixel is less than pThreshold is true, the pixel is set
 * to pThreshold, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdLessThanParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_LT_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                     const void * pSrc, int nSrcStep,
                     void * pDst, int nDstStep, NppiSize oSizeROI, 
                     const void * pThreshold, NppStreamContext nppStreamCtx); 

/** 
 * In place image less than threshold.
 * If for a comparison operations sourcePixel is less than pThreshold is true, the pixel is set
 * to pThreshold, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdLessThanParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_LT_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                       void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, 
                       const void * pThreshold, NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_less_than_operations */

/** 
 * \section image_threshold_value_operations Threshold Value Operations
 * @defgroup image_threshold_value_operations Threshold Value Operations
 * Replace thresholded image pixels with a value.
 *
 * \section CommonThresholdValueParameters Common parameters for nppiThreshold_Val non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold Host memory pointer to the threshold value(s) of the same data type.
 * \param pValue Host memory pointer to the threshold replacement value(s) of the same data type.
 * \param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image threshold value.
 * If for a comparison operations OP the predicate (sourcePixel OP pThreshold) is true, the pixel is set
 * to pValue, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_Val_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                     const void * pSrc, int nSrcStep,
                     void * pDst, int nDstStep, NppiSize oSizeROI, 
                     const void * pThreshold, const void * pvalue,
                     NppCmpOp eComparisonOperation, NppStreamContext nppStreamCtx); 

/** 
 * In place image threshold value.
 * If for a comparison operations OP the predicate (sourcePixel OP pThreshold) is true, the pixel is set
 * to pValue, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_Val_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                        void * SrcpDst, int nSrcDstStep, NppiSize oSizeROI, 
                        const void * pThreshold, const void * pvalue,
                        NppCmpOp eComparisonOperation, NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_value_operations */

/** 
 * \section image_threshold_greater_than_value_operations Threshold Greater Than Value Operations
 * @defgroup image_threshold_greater_than_value_operations Threshold Greater Than Value Operations
 * Replace image pixels greater than threshold with a value.
 *
 * \section CommonThresholdGreaterThanValueParameters Common parameters for nppiThreshold_GTVal non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold The threshold value.
 * \param pValue The threshold replacement value.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image greater than threshold value.
 * If for a comparison operations sourcePixel is greater than pThreshold is true, the pixel is set
 * to pValue, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdGreaterThanValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_GTVal_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                        const void * pSrc, int nSrcStep,
                        void * pDst, int nDstStep, NppiSize oSizeROI, 
                        const void * pThreshold, const void * pvalue,
                        NppStreamContext nppStreamCtx); 

/** 
 * In place image greater than threshold value.
 * If for a comparison operations sourcePixel is greater than pThreshold is true, the pixel is set
 * to pValue, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdGreaterThanValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_GTVal_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                          void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, 
                          const void * pThreshold, const void * pvalue,
                          NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_greater_than_value_operations */


/** 
 * \section image_fused_absdiff_threshold_greater_than_value_operations Fused AbsDiff Threshold Greater Than Value Operations
 * @defgroup image_fused_absdiff_threshold_greater_than_value_operations Fused AbsDiff Threshold Greater Than Value Operations
 * Replace image pixels greater than threshold with a value.
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc1 \ref source_image_pointer for non-inplace functions.
 * \param nSrc1Step \ref source_image_line_step for non-inplace functions.
 * \param pSrc2 \ref source_image_pointer to second source image for non-inplace functions.
 * \param nSrc2Step \ref source_image_line_step of second source image for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold The threshold value.
 * \param pValue The threshold replacement value.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image fused absdiff and greater than threshold value.
 * If for a comparison operations absdiff of sourcePixels is greater than pThreshold is true, the output pixel is set
 * to pValue, otherwise it is set to absdiff of sourcePixels.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiFusedAbsDiff_Threshold_GTVal_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                                     const void * pSrc1, int nSrc1Step,
                                     const void * pSrc2, int nSrc2Step,
                                     void * pDst, int nDstStep, NppiSize oSizeROI, 
                                     const void * pThreshold, const void * pvalue,
                                     NppStreamContext nppStreamCtx); 

/** 
 * In place fused absdiff image greater than threshold value.
 * If for a comparison operations absdiff of sourcePixels is greater than pThreshold is true, the output pixel is set
 * to pValue, otherwise it is set to absdiff of sourcePixels.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiFusedAbsDiff_Threshold_GTVal_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                                       void * pSrcDst, int nSrcDstStep,  
                                       const void * pSrc2, int nSrc2Step, NppiSize oSizeROI,
                                       const void * pThreshold, const void * pvalue,
                                       NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_fused_absdiff_threshold_greater_than_value_operations */

/** 
 * \section image_threshold_less_than_value_operations Threshold Less Than Value Operations
 * @defgroup image_threshold_less_than_value_operations Threshold Less Than Value Operations
 * Replace image pixels less than threshold with a value.
 *
 * \section CommonThresholdLessThanValueParameters Common parameters for nppiThreshold_LTVal non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThreshold The threshold value.
 * \param pValue The threshold replacement value.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image threshold less than value.
 * If for a comparison operations sourcePixel is less than pThreshold is true, the pixel is set
 * to pValue, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdLessThanValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_LTVal_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                        const void * pSrc, int nSrcStep,
                        void * pDst, int nDstStep, NppiSize oSizeROI, 
                        const void * pThreshold, const void * pvalue,
                        NppStreamContext nppStreamCtx); 

/** 
 * In place image threshold less than value.
 * If for a comparison operations sourcePixel is less than pThreshold is true, the pixel is set
 * to pValue, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdLessThanValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_LTVal_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                          void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, 
                          const void * pThreshold, const void * pvalue,
                          NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_less_than_value_operations */

/** 
 * \section image_threshold_less_than_value_greater_than_value_operations Threshold Less Than Value Or Greater Than Value Operations
 * @defgroup image_threshold_less_than_value_greater_than_value_operations Threshold Less Than Value Or Greater Than Value Operations
 * Replace image pixels less than thresholdLT or greater than thresholdGT with with valueLT or valueGT respectively.
 *
 * \section CommonThresholdLessThanValueGreaterThanValueParameters Common parameters for nppiThreshold_LTValGTVal non-inplace and inplace functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcDstType \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels. 
 * \param pSrcDst \ref in_place_image_pointer for inplace functions.
 * \param nSrcDstStep \ref in_place_image_line_step for inplace functions.
 * \param pSrc \ref source_image_pointer for non-inplace functions.
 * \param nSrcStep \ref source_image_line_step for non-inplace functions.
 * \param pDst \ref destination_image_pointer for non-inplace functions.
 * \param nDstStep \ref destination_image_line_step for non-inplace functions.
 * \param oSizeROI \ref roi_specification.
 * \param pThresholdLT Host memory pointer to the thresholdLT value(s) of the same data type.
 * \param pValueLT Host memory pointer to the thresholdLT replacement value(s) of the same data type.
 * \param pThresholdGT Host memory pointer to the thresholdGT value(s) of the same data type.
 * \param pValueGT Host memory pointer to the thresholdGT replacement value(s) of the same data type.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 *
 * @{
 *
 */


#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image threshold less than low value and greater than high value.
 * If for a comparison operations sourcePixel is less than pThresholdLT is true, the pixel is set
 * to pValueLT, else if sourcePixel is greater than pThresholdGT the pixel is set to pValueGT, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdLessThanValueGreaterThanValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_LTValGTVal_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                             const void * pSrc, int nSrcStep,
                             void * pDst, int nDstStep, NppiSize oSizeROI, 
                             const void * pThresholdLT, const void * pvalueLT,
                             const void * pThresholdGT, const void * pvalueGT,
                             NppStreamContext nppStreamCtx); 

/** 
 * In place image threshold less than low value and greater than high value.
 * If for a comparison operations sourcePixel is less than pThresholdLT is true, the pixel is set
 * to pValueLT, else if sourcePixel is greater than pThresholdGT the pixel is set to pValueGT, otherwise it is set to sourcePixel.
 * 
 * For common parameter descriptions, see \ref CommonThresholdLessThanValueGreaterThanValueParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiThreshold_LTValGTVal_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                               void * pSrcDst, int nSrcDstStep, NppiSize oSizeROI, 
                               const void * pThresholdLT, const void * pvalueLT,
                               const void * pThresholdGT, const void * pvalueGT,
                               NppStreamContext nppStreamCtx); 

#ifdef NPP_PLUS
}
#endif
/** @} image_threshold_less_than_value_greater_than_value_operations */

/** @} image_thresholding_operations */

/** 
 * \section image_comparison_operations Comparison Operations
 * @defgroup image_comparison_operations Comparison Operations
 * Compare the pixels of two images or one image and a constant value and create a binary result image. In case of multi-channel
 * image types, the condition must be fulfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type NPP_8U, NPP_CH_1. False is represented by 0, true by NPP_MAX_8U.
 *
 * @{
 *
 */


/** 
 * \section compare_images_operations Compare Images Operations
 * @defgroup compare_images_operations Compare Images Operations
 * Compare the pixels of two images and create a binary result image. In case of multi-channel
 * image types, the condition must be fulfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type NPP_8U, NPP_CH_1. False is represented by 0, true by NPP_MAX_8U.
 *
 * \section CommonCompareImagesParameters Common parameters for nppiCompare functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Imaage compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * 
 * For common parameter descriptions, see \ref CommonCompareImagesParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCompare_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,
                Npp8u * pDst,  int nDstStep, NppiSize oSizeROI, 
                NppCmpOp eComparisonOperation, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} compare_images_operations */

/** 
 * \section compare_image_with_constant_operations Compare Image With Constant Operations
 * @defgroup compare_image_with_constant_operations Compare Image With Constant Operations
 * Compare the pixels of an image with a constant value and create a binary result image. In case of multi-channel
 * image types, the condition must be fulfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type NPP_8U, NPP_CH_1. False is represented by 0, true by NPP_MAX_8U.
 *
 * \section CommonCompareImageWithConstantParameters Common parameters for nppiCompareC functions include:
 *
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstant Host memory pointer to constant value(s) of the same data type, one per color channel for multi-channel functions.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Image compare with constant value.
 * Compare pSrc's pixels with constant value. 
 * 
 * For common parameter descriptions, see \ref CommonCompareImageWithConstantParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCompareC_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                 const void * pSrc, int nSrcStep, void * pConstant,
                 Npp8u * pDst,  int nDstStep, NppiSize oSizeROI, 
                 NppCmpOp eComparisonOperation, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} compare_image_with_constant_operations */

/** 
 * \section compare_image_differences_with_epsilon_operations Compare Image Differences With Epsilon Operations
 * @defgroup compare_image_differences_with_epsilon_operations Compare Image Differences With Epsilon Operations
 * Compare the pixels value differences of two images with an epsilon value and create a binary result image. In case of multi-channel
 * image types, the condition must be fulfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type NPP_8U, NPP_CH_1. False is represented by 0, true by NPP_MAX_8U.
 *
 * \section CommonCompareImageDifferencesWithEpsilonParameters Common parameters for nppiCompareEqualEps functions include:
 *
 * Supported data types include NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nEpsilon epsilon tolerance value to compare to pixel absolute differences
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point image compare whether two images are equal within epsilon.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2 to determine whether they are equal with a difference of epsilon. 
 * 
 * For common parameter descriptions, see \ref CommonCompareImageDifferencesWithEpsilonParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCompareEqualEps_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,
                        Npp8u * pDst,  int nDstStep, NppiSize oSizeROI, 
                        Npp32f nEpsilon, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} compare_image_differences_with_epsilon_operations */


/** 
 * \section compare_image_difference_to_constant_with_epsilon_operations Compare Image Difference With Constant Within Epsilon Operations
 * @defgroup compare_image_difference_to_constant_with_epsilon_operations Compare Image Difference With Constant Within Epsilon Operations
 * Compare differences between image pixels and constant within an epsilon value and create a binary result image. In case of multi-channel
 * image types, the condition must be fulfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type NPP_8U, NPP_CH_1. False is represented by 0, true by NPP_MAX_8U.
 *
 * \section CommonCompareImageDifferenceWithConstantWithinEpsilonParameters Common parameters for nppiCompareEqualEpsC functions include:
 *
 * Supported data types include NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pConstants Host memory pointer to constant value(s) of the same data type, one per color channel for multi-channel image functions.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nEpsilon epsilon tolerance value to compare to per color channel pixel absolute differences
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point image compare whether image and constant are equal within epsilon.
 * Compare pSrc's pixels with constant value to determine whether they are equal within a difference of epsilon. 
 * 
 * For common parameter descriptions, see \ref CommonCompareImageDifferenceWithConstantWithinEpsilonParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiCompareEqualEpsC_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                         const void * pSrc, int nSrcStep, void * pConstant,
                         Npp8u * pDst,  int nDstStep, NppiSize oSizeROI, 
                         Npp32f nEpsilon, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} compare_image_difference_to_constant_within_epsilon_operations */

/** @} image_comparison_operations */

/** @} image_threshold_and_compare_operations */

#endif /* NV_NPPI_PLUS_THRESHOLD_AND_COMPARE_OPERATIONS_H */
