 /* Copyright 2010-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPS_PLUS_ARITHMETIC_AND_LOGICAL_OPERATIONS_H
#define NV_NPPS_PLUS_ARITHMETIC_AND_LOGICAL_OPERATIONS_H
 
/**
 * \file nppsPlus_arithmetic_and_logical_operations.h
 * Signal Arithmetic and Logical Operations.
 */

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 * \page signal_arithmetic_and_logical_operations Arithmetic and Logical Operations
 * @defgroup signal_arithmetic_and_logical_operations Arithmetic and Logical Operations
 * @ingroup npps
 * Functions that provide common arithmetic and logical operations.
 * @{
 *
 */

/** 
 * \section signal_arithmetic Arithmetic Operations
 * @defgroup signal_arithmetic Arithmetic Operations
 * The set of arithmetic operations for signal processing available in the library.
 * @{
 *
 */

/** 
 * \section signal_addc AddC
 * @defgroup signal_addc AddC
 * Adds a constant value to each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point in place signal add constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be added to each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddC_32f_I_Ctx(Npp32f nValue, Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal add constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be added to each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddC_32f_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point, in place signal add constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be added to each vector element
 * \param nLength Length of the vectors, number of items.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddC_64f_I_Ctx(Npp64f nValue, Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating pointsignal add constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be added to each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddC_64f_Ctx(const Npp64f * pSrc, Npp64f nValue, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal add constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be added to each vector element
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddC_ISfs_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, 
                  size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Signal add constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be added to each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddC_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, void * pDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_addc */

/** 
 * \section signal_addproductc AddProductC
 * @defgroup signal_addproductc AddProductC
 * Adds product of a constant and each sample of a source signal to the each sample of destination signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal add product of signal times constant to destination signal.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddProductC_32f_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_addproductc */

/** 
 * \section signal_mulc MulC
 * @defgroup signal_mulc MulC
 *
 * Multiplies each sample of a signal by a constant value.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal times constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_32f_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point in place signal times constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_32f_I_Ctx(Npp32f nValue, Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal times constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_64f_Ctx(const Npp64f * pSrc, Npp64f nValue, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point in place signal times constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_64f_I_Ctx(Npp64f nValue, Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Signal times constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to multiply each vector element by.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, void * pDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place signal times constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to multiply each vector element by.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_ISfs_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, 
                  size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal times constant with output converted to 16-bit signed integer.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_Low_32f16s_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp16s * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal times constant with output converted to 16-bit signed integer
 * with scaling and saturation of output result.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be multiplied by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_32f16s_Sfs_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal times constant with in place conversion to 64-bit signed integer
 * and with scaling and saturation of output result.
 * \param nValue Constant value to be multiplied by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMulC_64f64s_ISfs_Ctx(Npp64f nValue, Npp64s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_mulc */

/** 
 * \section signal_subc SubC
 * @defgroup signal_subc SubC
 *
 * Subtracts a constant from each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point in place signal subtract constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be subtracted from each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubC_32f_I_Ctx(Npp32f nValue, Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal subtract constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be subtracted from each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubC_32f_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point in place signal subtract constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be subtracted from each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubC_64f_I_Ctx(Npp64f nValue, Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal subtract constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be subtracted from each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubC_64f_Ctx(const Npp64f * pSrc, Npp64f nValue, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal subtract constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to subtract from each vector element.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubC_ISfs_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, 
                  size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Signal subtract constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to subtract from each vector element.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubC_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, void * pDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_subc */

/** 
 * \section signal_subcrev SubCRev
 * @defgroup signal_subcrev SubCRev
 *
 * Subtracts each sample of a signal from a constant.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Constant subtract signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to subtract each vector element from.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubCRev_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, void * pDst, 
                    size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place constant subtract signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to subtract each vector element from.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubCRev_ISfs_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, 
                     size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Floating point signal subtract from constant.
 * Supported data types include NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to subtract each vector element from.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubCRev_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, void * pDst, 
                size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Floating point in place signal subtract from constant.
 * Supported data types include NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to subtract each vector element from.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSubCRev_I_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, 
                  size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_subcrev */

/** 
 * \section signal_divc DivC
 * @defgroup signal_divc DivC
 *
 * Divides each sample of a signal by a constant.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal divided by constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be divided into each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivC_32f_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point in place signal divided by constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be divided into each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivC_32f_I_Ctx(Npp32f nValue, Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal divided by constant.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be divided into each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivC_64f_Ctx(const Npp64f * pSrc, Npp64f nValue, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point in place signal divided by constant.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be divided into each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivC_64f_I_Ctx(Npp64f nValue, Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Signal divided by constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to divide each vector element by.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivC_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, void * pDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * In place signal divided by constant, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to divide each vector element by.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivC_ISfs_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, 
                  size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_divc */

/** 
 * \section signal_divcrev DivCRev
 * @defgroup signal_divcrev DivCRev
 *
 * Divides a constant by each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 16-bit unsigned short signal divided by constant, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be divided by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivCRev_16u_Ctx(const Npp16u * pSrc, Npp16u nValue, Npp16u * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place 16-bit unsigned short signal constant divided by signal, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be divided by each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivCRev_16u_I_Ctx(Npp16u nValue, Npp16u * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point constant divided by signal.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be divided by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivCRev_32f_Ctx(const Npp32f * pSrc, Npp32f nValue, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point in place constant divided by signal.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be divided by each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivCRev_32f_I_Ctx(Npp32f nValue, Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point constant divided by signal.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be divided by each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivCRev_64f_Ctx(const Npp64f * pSrc, Npp64f nValue, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point in place constant divided by signal.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be divided by each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDivCRev_64f_I_Ctx(Npp64f nValue, Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_divcrev */

/** 
 * \section signal_add Add
 * @defgroup signal_add Add
 *
 * Sample by sample addition of two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal add signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal add signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 8-bit unsigned char signal add signal with 16-bit unsigned result,
 * then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_8u16u_Ctx(const Npp8u * pSrc1, const Npp8u * pSrc2, Npp16u * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal add signal with 32-bit floating point result,
 * then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_16s32f_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Signal add signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be added to signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
                void * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 16/32-bit signed short in place signal add signal with 32-bit signed integer results,
 * then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_16s32s_I_Ctx(const Npp16s * pSrc, Npp32s * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal add signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be added to signal1 elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAdd_ISfs_Ctx(NppDataType eSrcDstType, const void * pSrc1, void * pSrcDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_add */

/** 
 * \section signal_addproduct AddProduct
 * @defgroup signal_addproduct AddProduct
 *
 * Adds sample by sample product of two signals to the destination signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal add product of source signal times destination signal to destination signal,
 * then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddProduct_32f_Ctx(const Npp32f * pSrc1, const Npp32f * pSrc2, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal add product of source signal times destination signal to destination signal,
 * then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddProduct_64f_Ctx(const Npp64f * pSrc1, const Npp64f * pSrc2, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal add product of source signal1 times source signal2 to destination signal,
 * with scaling, then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddProduct_16s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed short signal add product of source signal1 times source signal2 to destination signal,
 * with scaling, then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddProduct_32s_Sfs_Ctx(const Npp32s * pSrc1, const Npp32s * pSrc2, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal add product of source signal1 times source signal2 to 32-bit signed integer destination signal,
 * with scaling, then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer. product of source1 and source2 signal elements to be added to destination elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAddProduct_16s32s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_addproduct */

/** 
 * \section signal_mul Mul
 * @defgroup signal_mul Mul
 *
 * Sample by sample multiplication the samples of two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal times signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal times signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, 
              size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 8-bit unsigned char signal times signal with 16-bit unsigned result,
 * then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_8u16u_Ctx(const Npp8u * pSrc1, const Npp8u * pSrc2, Npp16u * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal times signal with 32-bit floating point result,
 * then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_16s32f_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Integer signal times signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
                void * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 16-bit unsigned short signal times 16-bit signed short signal, scale, then clamp to 16-bit signed saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_16u16s_Sfs_Ctx(const Npp16u * pSrc1, const Npp16s * pSrc2, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal times signal, scale, then clamp to 32-bit signed saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_16s32s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal times signal, scale, then clamp to saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal2 elements to be multiplied by signal1 elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_Low_32s_Sfs_Ctx(const Npp32s * pSrc1, const Npp32s * pSrc2, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Integer in place signal times signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be multiplied by signal1 elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMul_ISfs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_mul */

/** 
 * \section signal_sub Sub
 * @defgroup signal_sub Sub
 *
 * Sample by sample subtraction of the samples of two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal subtract signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_16F, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSub_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal subtract signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSub_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, 
              size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal subtract 16-bit signed short signal,
 * then clamp and convert to 32-bit floating point saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSub_16s32f_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Integer signal subtract signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal1 elements to be subtracted from signal2 elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSub_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
                void * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Integer in place signal subtract signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal1 elements to be subtracted from signal2 elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSub_ISfs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_sub */

/**
 * \section signal_div Div
 * @defgroup signal_div Div
 *
 * Sample by sample division of the samples of two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Floating point signal divide signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Floating point in place signal divide signal, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Integer signal divide signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, void * pDst, 
                size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Integer in place signal divide signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_ISfs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal divided by 16-bit signed short signal, scale, then clamp to 16-bit signed short saturated value.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_32s16s_Sfs_Ctx(const Npp16s * pSrc1, const Npp32s * pSrc2, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_div */

/** 
 * \section signal_divround Div_Round
 * @defgroup signal_divround Div_Round
 *
 * Sample by sample division of the samples of two signals with rounding.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Integer signal divide signal, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer, signal1 divisor elements to be divided into signal2 dividend elements.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nRndMode various rounding modes.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_Round_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, void * pDst, 
                      size_t nLength, NppRoundMode nRndMode, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Integer in place signal divide signal, with scaling, rounding then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal1 divisor elements to be divided into signal2 dividend elements
 * \param nLength \ref length_specification.
 * \param nRndMode various rounding modes.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDiv_Round_ISfs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, 
                       size_t nLength, NppRoundMode nRndMode, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_divround */

/** 
 * \section signal_abs Abs
 * @defgroup signal_abs Abs
 *
 * Absolute value of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal absolute value.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAbs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal absolute value.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAbs_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_abs */

/** 
 * \section signal_square Sqr
 * @defgroup signal_square Sqr
 *
 * Squares each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal squared.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqr_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal squared.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqr_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal squared.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqr_64f_Ctx(const Npp64f * pSrc, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal squared.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqr_64f_I_Ctx(Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Integer signal squared, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqr_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, 
                size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Integer in place signal squared, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqr_ISfs_Ctx(NppDataType eSrcDstType, void * pSrcDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_square */

/** 
 * \section signal_sqrt Sqrt
 * @defgroup signal_sqrt Sqrt
 *
 * Square root of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal square root.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal square root.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal square root.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_64f_Ctx(const Npp64f * pSrc, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal square root.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_64f_I_Ctx(Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Integer signal square root, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_Sfs_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, 
                 size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Integer in place signal square root, scale, then clamp to saturated value.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_ISfs_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal square root, scale, then clamp to 16-bit signed integer saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_32s16s_Sfs_Ctx(const Npp32s * pSrc, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 64-bit signed integer signal square root, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_64s_Sfs_Ctx(const Npp64s * pSrc, Npp64s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 64-bit signed integer signal square root, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_64s_ISfs_Ctx(Npp64s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 64-bit signed integer signal square root, scale, then clamp to 16-bit signed integer saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSqrt_64s16s_Sfs_Ctx(const Npp64s * pSrc, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_sqrt */

/** 
 * \section signal_cuberoot Cubrt
 * @defgroup signal_cuberoot Cubrt
 *
 * Cube root of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal cube root.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsCubrt_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal cube root, scale, then clamp to 16-bit signed integer saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsCubrt_32s16s_Sfs_Ctx(const Npp32s * pSrc, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_cuberoot */

/** 
 * \section signal_exp Exp
 * @defgroup signal_exp Exp
 *
 * E raised to the power of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal exponent.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal exponent.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_64f_Ctx(const Npp64f * pSrc, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal exponent with 64-bit floating point result.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_32f64f_Ctx(const Npp32f * pSrc, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal exponent.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal exponent.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_64f_I_Ctx(Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal exponent, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_16s_Sfs_Ctx(const Npp16s * pSrc, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal exponent, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_32s_Sfs_Ctx(const Npp32s * pSrc, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 64-bit signed integer signal exponent, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_64s_Sfs_Ctx(const Npp64s * pSrc, Npp64s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal exponent, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_16s_ISfs_Ctx(Npp16s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal exponent, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_32s_ISfs_Ctx(Npp32s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 64-bit signed integer signal exponent, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsExp_64s_ISfs_Ctx(Npp64s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_exp */

/** 
 * \section signal_ln Ln
 * @defgroup signal_ln Ln
 *
 * Natural logarithm of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal natural logarithm.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal natural logarithm.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_64f_Ctx(const Npp64f * pSrc, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal natural logarithm with 32-bit floating point result.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_64f32f_Ctx(const Npp64f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal natural logarithm.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal natural logarithm.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_64f_I_Ctx(Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal natural logarithm, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_16s_Sfs_Ctx(const Npp16s * pSrc, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal natural logarithm, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_32s_Sfs_Ctx(const Npp32s * pSrc, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal natural logarithm, scale, then clamp to 16-bit signed short saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_32s16s_Sfs_Ctx(const Npp32s * pSrc, Npp16s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal natural logarithm, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_16s_ISfs_Ctx(Npp16s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal natural logarithm, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLn_32s_ISfs_Ctx(Npp32s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_ln */

/** 
 * \section signal_10log10 10Log10
 * @defgroup signal_10log10 10Log10
 *
 * Ten times the decimal logarithm of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit signed integer signal 10 times base 10 logarithm, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
npps10Log10_32s_Sfs_Ctx(const Npp32s * pSrc, Npp32s * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * 32-bit signed integer signal 10 times base 10 logarithm, scale, then clamp to saturated value.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
npps10Log10_32s_ISfs_Ctx(Npp32s * pSrcDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_10log10 */

/** 
 * \section signal_sumln SumLn
 * @defgroup signal_sumln SumLn
 *
 * Sums up the natural logarithm of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for 32f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_32f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsSumLnGetBufferSize_32f_Ctx(size_t nLength, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal sum natural logarithm.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSumLn_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device scratch buffer size (in bytes) for 64f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_64f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsSumLnGetBufferSize_64f_Ctx(size_t nLength, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal sum natural logarithm.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSumLn_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device scratch buffer size (in bytes) for 32f64f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_32f64f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsSumLnGetBufferSize_32f64f_Ctx(size_t nLength, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * 32-bit flaoting point input, 64-bit floating point output signal sum natural logarithm.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSumLn_32f64f_Ctx(const Npp32f * pSrc, size_t nLength, Npp64f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device scratch buffer size (in bytes) for 16s32f SumLn.
 * This primitive provides the correct buffer size for nppsSumLn_16s32f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsSumLnGetBufferSize_16s32f_Ctx(size_t nLength, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer input, 32-bit floating point output signal sum natural logarithm.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSumLn_16s32f_Ctx(const Npp16s * pSrc, size_t nLength, Npp32f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_sumln */

/** 
 * \section signal_inversetan Arctan
 * @defgroup signal_inversetan Arctan
 *
 * Inverse tangent of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal inverse tangent.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsArctan_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal inverse tangent.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsArctan_64f_Ctx(const Npp64f * pSrc, Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal inverse tangent.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsArctan_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal inverse tangent.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsArctan_64f_I_Ctx(Npp64f * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_inversetan */

/** 
 * \section signal_normalize Normalize
 * @defgroup signal_normalize Normalize
 *
 * Normalize each sample of a real or complex signal using offset and division operations.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal normalize.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param vSub value subtracted from each signal element before division
 * \param vDiv divisor of post-subtracted signal element dividend
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormalize_32f_Ctx(const Npp32f * pSrc, Npp32f * pDst, size_t nLength, Npp32f vSub, Npp32f vDiv, NppStreamContext nppStreamCtx);

/** 
 * 64-bit floating point signal normalize.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param vSub value subtracted from each signal element before division
 * \param vDiv divisor of post-subtracted signal element dividend
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormalize_64f_Ctx(const Npp64f * pSrc, Npp64f * pDst, size_t nLength, Npp64f vSub, Npp64f vDiv, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short signal normalize, scale, then clamp to saturated value.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param vSub value subtracted from each signal element before division
 * \param vDiv divisor of post-subtracted signal element dividend
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormalize_16s_Sfs_Ctx(const Npp16s * pSrc, Npp16s * pDst, size_t nLength, Npp16s vSub, int vDiv, int nScaleFactor, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_normalize */

/** 
 * \section signal_cauchy Cauchy, CauchyD, and CauchyDD2
 * @defgroup signal_cauchy Cauchy, CauchyD, and CauchyDD2
 *
 * Determine Cauchy robust error function and its first and second derivatives for each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * 32-bit floating point signal Cauchy error calculation.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nParam constant used in Cauchy formula
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsCauchy_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, Npp32f nParam, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal Cauchy first derivative.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nParam constant used in Cauchy formula
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsCauchyD_32f_I_Ctx(Npp32f * pSrcDst, size_t nLength, Npp32f nParam, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating point signal Cauchy first and second derivatives.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pD2FVal \ref source_signal_pointer. This signal contains the second derivative
 *      of the source signal.
 * \param nLength \ref length_specification.
 * \param nParam constant used in Cauchy formula
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsCauchyDD2_32f_I_Ctx(Npp32f * pSrcDst, Npp32f * pD2FVal, size_t nLength, Npp32f nParam, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_cauchy */

/** @} signal_arithmetic_operations */

/** 
 * \section signal_logical_and_shift_operations Logical And Shift Operations
 * @defgroup signal_logical_and_shift_operations Logical And Shift Operations
 * The set of logical and shift operations for signal processing available in the library.
 * @{
 *
 */

/** 
 * \section signal_andc AndC
 * @defgroup signal_andc AndC
 *
 * Bitwise AND of a constant and each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal logical AND with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be logical anded with each vector element.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAndC_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant, 
             void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal logincal AND with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be logical anded with each vector element.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAndC_I_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_andc */

/** 
 * \section signal_and And
 * @defgroup signal_and And
 *
 * Sample by sample bitwise AND of samples from two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal logical AND with signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be logical anded with signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAnd_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal logical AND with signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be logical anded with signal1 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAnd_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_and */

/** 
 * \section signal_orc OrC
 * @defgroup signal_orc OrC
 *
 * Bitwise OR of a constant and each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal logical OR with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be logical ored with each vector element.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsOrC_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant , 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal logical OR with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be logical ored with each vector element.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsOrC_I_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_orc */

/** 
 * \section signal_or Or
 * @defgroup signal_or Or
 *
 * Sample by sample bitwise OR of the samples from two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal logical OR with signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be ored with signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsOr_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
           void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal logical OR with signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be ored with signal1 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsOr_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_or */

/** 
 * \section signal_xorc XorC
 * @defgroup signal_xorc XorC
 *
 * Bitwise XOR of a constant and each sample of a signal.
 *
 * @{
 *
 */


#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal logical XOR with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be logical xored with each vector element.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsXorC_Ctx(NppDataType eSrcDstType, const void * pSrc, const void * pConstant , 
             void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal logical XOR with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param pConstant Host memory pointer to eSrcDstType Constant value to be logical xored with each vector element.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsXorC_I_Ctx(NppDataType eSrcDstType, const void * pConstant, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_xorc */

/** 
 * \section signal_xor Xor
 * @defgroup signal_xor Xor
 *
 * Sample by sample bitwise XOR of the samples from two signals.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal logical XOR with signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer. signal2 elements to be xored with signal1 elements
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsXor_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, 
            void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal logical XOR with signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer. signal2 elements to be xored with signal1 elements
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsXor_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_xor */

/** 
 * \section signal_not Not
 * @defgroup signal_not Not
 *
 * Bitwise NOT of each sample of a signal.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * logical NOT signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNot_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place logical NOT signal.
 * Supported data types include NPP_8U, NPP_16U, NPP_32U. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNot_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_not */

/** 
 * \section signal_lshiftc LShiftC
 * @defgroup signal_lshiftc LShiftC
 *
 * Left shifts the bits of each sample of a signal by a constant amount.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal left shift with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be used to left shift each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLShiftC_Ctx(NppDataType eSrcDstType, const void * pSrc, int nValue, 
                void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal left shift with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be used to left shift each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsLShiftC_I_Ctx(NppDataType eSrcDstType, int nValue, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_lshiftc */

/** 
 * \section signal_rshiftc RShiftC
 * @defgroup signal_rshiftc RShiftC
 *
 * Right shifts the bits of each sample of a signal by a constant amount.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal right shift with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param nValue Constant value to be used to right shift each vector element
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsRShiftC_Ctx(NppDataType eSrcDstType, const void * pSrc, int nValue, 
                void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place signal right shift with constant.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nValue Constant value to be used to right shift each vector element
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsRShiftC_I_Ctx(NppDataType eSrcDstType, int nValue, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_rshiftc */

/** @} signal_logical_and_shift_operations */

/** @} signal_arithmetic_and_logical_operations */

#endif /* NV_NPPS_PLUS_ARITHMETIC_AND_LOGICAL_OPERATIONS_H */
