 /* Copyright 2010-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPS_PLUS_INITIALIZATION_H
#define NV_NPPS_PLUS_INITIALIZATION_H
 
/**
 * \file nppsPlus_initialization.h
 * NPP Signal Processing Functionality.
 */
 

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"
 
/** 
 * \page signal_initialization Initialization
 * @defgroup signal_initialization Initialization
 * @ingroup npps
 * Functions that provide functionality of initialization signal like: set, zero or copy other signal.
 * @{
 */

/** 
 * \section signal_set Set
 * @defgroup signal_set Set
 * The set of set initialization operations available in the library.
 * @{
 *
 */

/** @name Set 
 * Set methods for 1D vectors of various types. The copy methods operate on vector data given
 * as a pointer to the underlying data-type (e.g. 8-bit vectors would
 * be passed as pointers to Npp8u type) and length of the vectors, i.e. the number of items.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Vector set method.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pValue Host memory pointer to eSrcDstType Value used to initialize the vector pDst.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSet_Ctx(NppDataType eSrcDstType, const void * pValue, void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} end of Signal Set */
/** @} signal_set */

/** 
 * \section signal_copy Copy
 * @defgroup signal_copy Copy
 * The set of copy initialization operations available in the library.
 * @{
 *
 */

/** @name Copy
 * Copy methods for various type signals. Copy methods operate on
 * signal data given as a pointer to the underlying data-type (e.g. 8-bit
 * vectors would be passed as pointers to Npp8u type) and length of the
 * vectors, i.e. the number of items. 
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Vector copy method
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsCopy_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} end of Copy */

/** @} signal_copy */

/** 
 * \section signal_zero Zero
 * @defgroup signal_zero Zero
 * The set of zero initialization operations available in the library.
 * @{
 *
 */

/** @name Zero
 * Set signals to zero.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 8-bit unsigned char, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_8u_Ctx(Npp8u * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit unsigned integer, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_16u_Ctx(Npp16u * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 16-bit integer, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_16s_Ctx(Npp16s * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_32s_Ctx(Npp32s * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 32-bit float, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_32f_Ctx(Npp32f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit long long integer, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_64s_Ctx(Npp64s * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * 64-bit double, vector zero method.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsZero_64f_Ctx(Npp64f * pDst, size_t nLength, NppStreamContext nppStreamCtx);

}
#endif
/** @} end of Zero */

/** @} signal_zero */

/** @} signal_initialization */

#endif /* NV_NPPS_PLUS_INITIALIZATION_H */
