 /* Copyright 2010-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPS_PLUS_STATISTICS_FUNCTIONS_H
#define NV_NPPS_PLUS_STATISTICS_FUNCTIONS_H
 
/**
 * \file nppsPlus_statistics_functions.h
 * NPP Signal Processing Functionality.
 */

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 *  \page signal_statistical_functions Statistical Functions
 *  @defgroup signal_statistical_functions Statistical Functions
 *  @ingroup npps
 * Functions that provide global signal statistics like: sum, mean, standard
 * deviation, min, max, etc.
 *
 * @{
 *
 */

/** 
 * \section signal_min_every_or_max_every MinEvery And MaxEvery Functions
 * @defgroup signal_min_every_or_max_every MinEvery And MaxEvery Functions
 * Performs the min or max operation on the samples of a signal.
 *
 * @{  
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * In place min value for each pair of elements.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinEvery_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * In place max value for each pair of elements.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxEvery_I_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pSrcDst, size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/**
 * @} signal_min_every_or_max_every
 *
 */

/** 
 * \section signal_sum Sum
 * @defgroup signal_sum Sum
 * Performs the sum operation on the samples of a signal.
 * @{  
 *
 */
 
#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsSum.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType destination \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsSumGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector sum method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pSum Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsSumGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSum_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pSum, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit double vector sum method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pSum Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsSumGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSum_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pSum, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit short vector sum with integer scaling method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pSum Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsSumGetBufferSize to determine the minium number of bytes required.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSum_16s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pSum, int nScaleFactor, 
                    Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector sum with integer scaling method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pSum Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsSumGetBufferSize to determine the minium number of bytes required.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSum_32s_Sfs_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pSum, int nScaleFactor, 
                    Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit integer vector sum (32bit) with integer scaling method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pSum Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsSumGetBufferSize to determine the minium number of bytes required.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsSum_16s32s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp32s * pSum, int nScaleFactor,
                       Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);
}
#endif
/** @} signal_sum */


/** 
 * \section signal_max Maximum
 * @defgroup signal_max Maximum
 * Performs the maximum operation on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMax.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMaxGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                         size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector max method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMax_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMax, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector max method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMax_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMax, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit float vector max method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMax_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pMax, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector max method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMax_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pMax, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMaxIndx.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMaxIndxGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                             size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector max index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pIndx Pointer to the index value of the first maximum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxIndx_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMax, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector max index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pIndx Pointer to the index value of the first maximum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxIndx_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMax, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit float vector max index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pIndx Pointer to the index value of the first maximum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxIndx_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pMax, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector max index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMax Pointer to the output result.
 * \param pIndx Pointer to the index value of the first maximum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxIndx_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pMax, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMaxAbs.
 * Supported data types include NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMaxAbsGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, 
                            size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector max absolute method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMaxAbs Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxAbsGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxAbs_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMaxAbs, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector max absolute method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMaxAbs Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxAbsGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxAbs_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMaxAbs, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMaxAbsIndx.
 * Supported data types include NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMaxAbsIndxGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, 
                                size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector max absolute index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMaxAbs Pointer to the output result.
 * \param pIndx Pointer to the index value of the first maximum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxAbsIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxAbsIndx_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMaxAbs, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector max absolute index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMaxAbs Pointer to the output result.
 * \param pIndx Pointer to the index value of the first maximum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaxAbsIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaxAbsIndx_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMaxAbs, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_max */

/** 
 * \section signal_min Minimum
 * @defgroup signal_min Minimum
 * Performs the minimum operation on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMin.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMinGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                         size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector min method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMin_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMin, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector min method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMin_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMin, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector min method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMin_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pMin, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit integer vector min method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMin_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pMin, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMinIndx.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMinIndxGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                             size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector min index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pIndx Pointer to the index value of the first minimum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinIndx_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMin, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector min index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pIndx Pointer to the index value of the first minimum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinIndx_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMin, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit float vector min index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pIndx Pointer to the index value of the first minimum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinIndx_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pMin, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector min index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the output result.
 * \param pIndx Pointer to the index value of the first minimum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinIndx_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pMin, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMinAbs.
 * Supported data types include NPP_16S, NPP_32S. 
 * \param eDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMinAbsGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                            size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector min absolute method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMinAbs Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinAbsGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinAbs_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMinAbs, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector min absolute method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMinAbs Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinAbsGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinAbs_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMinAbs, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMinAbsIndx.
 * Supported data types include NPP_16S, NPP_32S. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMinAbsIndxGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, 
                                size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 16-bit integer vector min absolute index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMinAbs Pointer to the output result.
 * \param pIndx Pointer to the index value of the first minimum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinAbsIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinAbsIndx_16s_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMinAbs, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector min absolute index method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMinAbs Pointer to the output result.
 * \param pIndx Pointer to the index value of the first minimum element.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinAbsIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinAbsIndx_32s_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMinAbs, int * pIndx, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_min */

/** 
 * \section signal_mean Mean
 * @defgroup signal_mean Mean
 * Performs the mean operation on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMean.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMeanGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, 
                          size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector mean method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMean_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pMean, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit double vector mean method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMean_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pMean, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit short vector mean with integer scaling method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanGetBufferSize to determine the minium number of bytes required.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMean_16s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMean, int nScaleFactor, 
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 32-bit integer vector mean with integer scaling method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanGetBufferSize to determine the minium number of bytes required.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMean_32s_Sfs_Ctx(const Npp32s * pSrc, size_t nLength, Npp32s * pMean, int nScaleFactor, 
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_mean */

/** 
 * \section signal_standard_deviation Standard Deviation
 * @defgroup signal_standard_deviation Standard Deviation
 * Calculates the standard deviation for the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsStdDev.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsStdDevGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                            size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector standard deviation method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pStdDev Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsStdDev_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pStdDev, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector standard deviation method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pStdDev Pointer to the output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsStdDev_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pStdDev, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit float vector standard deviation method (return value is 32-bit)
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pStdDev Pointer to the output result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsStdDev_16s32s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp32s * pStdDev, int nScaleFactor, 
                          Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit float vector standard deviation method (return value is also 16-bit)
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pStdDev Pointer to the output result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsStdDev_16s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pStdDev, int nScaleFactor, 
                       Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_standard_deviation */

/** 
 * \section signal_mean_and_standard_deviation Mean And Standard Deviation
 * @defgroup signal_mean_and_standard_deviation Mean And Standard Deviation
 * Performs the mean and calculates the standard deviation for the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppsMeanStdDev.
 * Supported data types include NPP_16S, NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType output output \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus 
nppsMeanStdDevGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, 
                                size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector mean and standard deviation method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output mean value.
 * \param pStdDev Pointer to the output standard deviation value.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMeanStdDev_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pMean, Npp32f * pStdDev, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector mean and standard deviation method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output mean value.
 * \param pStdDev Pointer to the output standard deviation value.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMeanStdDev_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pMean, Npp64f * pStdDev, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit float vector mean and standard deviation method (return values are 32-bit)
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output mean value.
 * \param pStdDev Pointer to the output standard deviation value.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMeanStdDev_16s32s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp32s * pMean, Npp32s * pStdDev, int nScaleFactor, 
                              Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit float vector mean and standard deviation method (return values are also 16-bit)
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMean Pointer to the output mean value.
 * \param pStdDev Pointer to the output standard deviation value.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMeanStdDevGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMeanStdDev_16s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp16s * pMean, Npp16s * pStdDev, int nScaleFactor, 
                           Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_mean_and_standard_deviation */

/** 
 * \section signal_min_max Minimum Maximum
 * @defgroup signal_min_max Minimum Maximum
 * Performs the maximum and the minimum operation on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsMinMax.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32U, NPP_32F, NPP_64F. 
 * \param eDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsMinMaxGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * Vector min and max method
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32U, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the min output result.
 * \param pMax Pointer to the max output result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinMaxGetBufferSize_8u to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinMax_Ctx(NppDataType eSrcDstType, const void * pSrc, size_t nLength, void * pMin, void * pMax, 
               Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsMinMaxIndx_8u.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32S, NPP_32U, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsMinMaxIndxGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * Vector min and max with indices method
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pMin Pointer to the min output result.
 * \param pMinIndx Pointer to the index of the first min value.
 * \param pMax Pointer to the max output result.
 * \param pMaxIndx Pointer to the index of the first max value.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMinMaxIndxGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMinMaxIndx_Ctx(NppDataType eSrcDstType, const void * pSrc, size_t nLength, void * pMin, int * pMinIndx, void * pMax, int * pMaxIndx,
                   Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_min_max */

/** 
 * \section signal_infinity_norm Infinity Norm
 * @defgroup signal_infinity_norm Infinity Norm
 * Performs the infinity norm on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsNorm_Inf.
 * Supported data types include NPP_32S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormInfGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector C norm method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_Inf_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pNorm,
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector C norm method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_Inf_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pNorm,
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsNorm_Inf_16s32f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormInfGetBufferSize_16s32f_Ctx(size_t nLength,  size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector C norm method, return value is 32-bit float.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_Inf_16s32f_Ctx(const Npp16s * pSrc, size_t nLength, Npp32f * pNorm,
                        Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector C norm method, return value is 32-bit signed integer.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_Inf_16s32s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp32s * pNorm, int nScaleFactor,
                            Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_infinity_norm */

/** 
 * \section signal_L1_norm L1 Norm
 * @defgroup signal_L1_norm L1 Norm
 * Performs the L1 norm on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsNorm_L1.
 * Supported data types include NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormL1GetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector L1 norm method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L1_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pNorm,
                    Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector L1 norm method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L1_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pNorm,
                    Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector L1 norm method, return value is 32-bit float.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the L1 norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L1_16s32f_Ctx(const Npp16s * pSrc, size_t nLength, Npp32f * pNorm,
                       Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector L1 norm method, return value is 32-bit signed integer.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L1_16s32s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp32s * pNorm, int nScaleFactor,
                           Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector L1 norm method, return value is 64-bit signed integer.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L1_16s64s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp64s * pNorm, int nScaleFactor,
                           Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_L1_norm */

/** 
 * \section signal_L2_norm L2 Norm
 * @defgroup signal_L2_norm L2 Norm
 * Performs the L2 norm on the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsNorm_L2.
 * Supported data types include NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormL2GetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float vector L2 norm method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L2_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pNorm, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float vector L2 norm method
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L2_64f_Ctx(const Npp64f * pSrc, size_t nLength, Npp64f * pNorm,
                    Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector L2 norm method, return value is 32-bit float.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L2_16s32f_Ctx(const Npp16s * pSrc, size_t nLength, Npp32f * pNorm,
                       Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector L2 norm method, return value is 32-bit signed integer.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L2_16s32s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp32s * pNorm, int nScaleFactor,
                           Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer vector L2 Square norm method, return value is 64-bit signed integer.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormL2SqrGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNorm_L2Sqr_16s64s_Sfs_Ctx(const Npp16s * pSrc, size_t nLength, Npp64s * pNorm, int nScaleFactor,
                              Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_L2_norm */

/** 
 * \section signal_infinity_norm_diff Infinity Norm Diff
 * @defgroup signal_infinity_norm_diff Infinity Norm Diff
 * Performs the infinity norm on the samples of two input signals' difference.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsNormDiff_Inf.
 * Supported data types include NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormDiffInfGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float C norm method on two vectors' difference
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_Inf_32f_Ctx(const Npp32f * pSrc1, const Npp32f * pSrc2, size_t nLength, Npp32f * pNorm,
                         Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float C norm method on two vectors' difference
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_Inf_64f_Ctx(const Npp64f * pSrc1, const Npp64f * pSrc2, size_t nLength, Npp64f * pNorm,
                         Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);


/** 
 * 16-bit signed short integer C norm method on two vectors' difference, return value is 32-bit float.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_Inf_16s32f_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp32f * pNorm,
                            Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer C norm method on two vectors' difference, return value is 32-bit signed integer.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffInfGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_Inf_16s32s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp32s * pNorm, int nScaleFactor,
                                Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_infinity_norm_diff */

/** 
 * \section signal_L1_norm_diff L1 Norm Diff
 * @defgroup signal_L1_norm_diff L1 Norm Diff
 * Performs the L1 norm on the samples of two input signals' difference.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsNormDiff_L1.
 * Supported data types include NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormDiffL1GetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float L1 norm method on two vectors' difference
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L1_32f_Ctx(const Npp32f * pSrc1, const Npp32f * pSrc2, size_t nLength, Npp32f * pNorm,
                        Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float L1 norm method on two vectors' difference
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L1_64f_Ctx(const Npp64f * pSrc1, const Npp64f * pSrc2, size_t nLength, Npp64f * pNorm,
                        Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer L1 norm method on two vectors' difference, return value is 32-bit float.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the L1 norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L1_16s32f_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp32f * pNorm,
                           Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer L1 norm method on two vectors' difference, return value is 32-bit signed integer.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer..
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L1_16s32s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp32s * pNorm, int nScaleFactor,
                               Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer L1 norm method on two vectors' difference, return value is 64-bit signed integer.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL1GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L1_16s64s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp64s * pNorm, int nScaleFactor,
                               Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_L1_norm_diff */

/** 
 * \section signal_L2_norm_diff L2 Norm Diff
 * @defgroup signal_L2_norm_diff L2 Norm Diff
 * Performs the L2 norm on the samples of two input signals' difference.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsNormDiff_L2.
 * Supported data types include NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType output \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsNormDiffL2GetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * 32-bit float L2 norm method on two vectors' difference
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L2_32f_Ctx(const Npp32f * pSrc1, const Npp32f * pSrc2, size_t nLength, Npp32f * pNorm,
                        Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 64-bit float L2 norm method on two vectors' difference
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L2_64f_Ctx(const Npp64f * pSrc1, const Npp64f * pSrc2, size_t nLength, Npp64f * pNorm,
                        Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer L2 norm method on two vectors' difference, return value is 32-bit float.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L2_16s32f_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp32f * pNorm,
                           Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);


/** 
 * 16-bit signed short integer L2 norm method on two vectors' difference, return value is 32-bit signed integer.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L2_16s32s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp32s * pNorm, int nScaleFactor,
                               Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Integer L2 Square norm method on two vectors' difference, return value is eDstType.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pNorm Pointer to the norm result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsNormDiffL2GetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsNormDiff_L2Sqr_16s64s_Sfs_Ctx(const Npp16s * pSrc1, const Npp16s * pSrc2, size_t nLength, Npp64s * pNorm, int nScaleFactor,
                                  Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_l2_norm_diff */

/** 
 * \section signal_dot_product Dot Product
 * @defgroup signal_dot_product Dot Product
 * Performs the dot product operation on the samples of two input signals.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device-buffer size (in bytes) for nppsDotProd.
 * Supported data types include NPP_16S, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType output \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsDotProdGetBufferSize_Ctx(NppDataType eDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * dot product method, return value is eDstType.
 * Supported source data types include NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcType source \ref signal_data_type.
 * Supported output data types include NPP_16S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eDstType output data type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDp Device memory pointer to the dot product result of type eDstType.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsDotProdGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDotProd_Ctx(NppDataType eSrcType, NppDataType eDstType, const void * pSrc1, const void * pSrc2, size_t nLength, void * pDp,
                Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Integer dot product method, return value is eDstType.
 * Supported source data types include NPP_16S, NPP_32S. 
 * \param eSrcType source \ref signal_data_type.
 * Supported output data types include NPP_16S, NPP_32S. 
 * \param eDstType output data type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDp Device memory pointer to the dot product result of type eDstType.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsDotProdGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDotProd_Sfs_Ctx(NppDataType eSrcType, NppDataType eDstType, const void * pSrc1, const void * pSrc2, size_t nLength, void * pDp, int nScaleFactor, 
                    Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * Device-buffer size (in bytes) for nppsDotProd_16s32s32s_Sfs.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsDotProdGetBufferSize_16s32s32s_Sfs_Ctx(size_t nLength,  size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer and 32-bit signed integer dot product method, return value is 32-bit signed integer.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDp Pointer to the dot product result.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsDotProdGetBufferSize_16s32s32s_Sfs to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsDotProd_16s32s32s_Sfs_Ctx(const Npp16s * pSrc1, const Npp32s * pSrc2, size_t nLength, Npp32s * pDp, int nScaleFactor, 
                              Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_dot_product */

/** 
 * \section signal_count_in_range Count In Range
 * @defgroup signal_count_in_range Count In Range
 * Calculates the number of elements from specified range in the samples of a signal.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * Device-buffer size (in bytes) for nppsCountInRange_32s.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsCountInRangeGetBufferSize_32s_Ctx(size_t nLength,  size_t * hpBufferSize, NppStreamContext nppStreamCtx);

}
#endif

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * Computes the number of elements whose values fall into the specified range on a 32-bit signed integer array.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pCounts Pointer to the number of elements.
 * \param nLowerBound Lower bound of the specified range.
 * \param nUpperBound Upper bound of the specified range.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsCountInRangeGetBufferSize_32s to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus
nppsCountInRange_32s_Ctx(const Npp32s * pSrc, size_t nLength, int * pCounts, Npp32s nLowerBound, Npp32s nUpperBound,
                         Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_count_in_range */

/** 
 * \section signal_count_zero_crossings Count Zero Crossings
 * @defgroup signal_count_zero_crossings Count Zero Crossings
 *
 * @{
 * Calculates the number of zero crossings in a signal.
 */

#ifdef NPP_PLUS
namespace nppPlusV {
/** 
 * Device-buffer size (in bytes) for nppsZeroCrossing_16s32f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsZeroCrossingGetBufferSize_16s32f_Ctx(size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * 16-bit signed short integer zero crossing method, return value is 32-bit floating point.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pValZC Pointer to the output result.
 * \param tZCType Type of the zero crossing measure: nppZCR, nppZCXor or nppZCC.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsZeroCrossingGetBufferSize_16s32f to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus
nppsZeroCrossing_16s32f_Ctx(const Npp16s * pSrc, size_t nLength, Npp32f * pValZC, NppsZCType tZCType,
                            Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsZeroCrossing_32f.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsZeroCrossingGetBufferSize_32f_Ctx(size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

/** 
 * 32-bit floating-point zero crossing method, return value is 32-bit floating point.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pValZC Pointer to the output result.
 * \param tZCType Type of the zero crossing measure: nppZCR, nppZCXor or nppZCC.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsZeroCrossingGetBufferSize_32f to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus
nppsZeroCrossing_32f_Ctx(const Npp32f * pSrc, size_t nLength, Npp32f * pValZC, NppsZCType tZCType,
                         Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

}
#endif
/** @} signal_count_zero_crossings */

/** 
 * \section signal_maximum_error MaximumError
 * @defgroup signal_maximum_error MaximumError
 * Primitives for computing the maximum error between two signals.
 * Given two signals \f$pSrc1\f$ and \f$pSrc2\f$ both with length \f$N\f$, 
 * the maximum error is defined as the largest absolute difference between the corresponding
 * elements of two signals.
 *
 * If the signal is in complex format, the absolute value of the complex number is used.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Maximum error method.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the error result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaximumErrorGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaximumError_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, size_t nLength, Npp64f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsMaximumError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsMaximumErrorGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** 
 * \section signal_average_error AverageError
 * @defgroup signal_average_error AverageError
 * Primitives for computing the Average error between two signals.
 * Given two signals \f$pSrc1\f$ and \f$pSrc2\f$ both with length \f$N\f$, 
 * the average error is defined as
 * \f[Average Error = \frac{1}{N}\sum_{n=0}^{N-1}\left|pSrc1(n) - pSrc2(n)\right|\f]
 *
 * If the signal is in complex format, the absolute value of the complex number is used.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Average error method.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the error result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsAverageErrorGetBufferSize to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAverageError_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, size_t nLength, Npp64f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsAverageError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsAverageErrorGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** 
 * \section signal_maximum_relative_error MaximumRelativeError
 * @defgroup signal_maximum_relative_error MaximumRelativeError
 * Primitives for computing the MaximumRelative error between two signals.
 * Given two signals \f$pSrc1\f$ and \f$pSrc2\f$ both with length \f$N\f$, 
 * the maximum relative error is defined as
 * \f[MaximumRelativeError = max{\frac{\left|pSrc1(n) - pSrc2(n)\right|}{max(\left|pSrc1(n)\right|, \left|pSrc2(n)\right|)}}\f]
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * MaximumRelative error method.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the error result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsMaximumRelativeErrorGetBufferSize_8u to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsMaximumRelativeError_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, size_t nLength, Npp64f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsMaximumRelativeError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsMaximumRelativeErrorGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** 
 * \section signal_average_relative_error AverageRelativeError
 * @defgroup signal_average_relative_error AverageRelativeError
 * Primitives for computing the AverageRelative error between two signals.
 * Given two signals \f$pSrc1\f$ and \f$pSrc2\f$ both with length \f$N\f$, 
 * the average relative error is defined as
 * \f[AverageRelativeError = \frac{1}{N}\sum_{n=0}^{N-1}\frac{\left|pSrc1(n) - pSrc2(n)\right|}{max(\left|pSrc1(n)\right|, \left|pSrc2(n)\right|)}\f]
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * AverageRelative error method.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc1 \ref source_signal_pointer.
 * \param pSrc2 \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst Pointer to the error result.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppsAverageRelativeErrorGetBufferSize_8u to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsAverageRelativeError_Ctx(NppDataType eSrcDstType, const void * pSrc1, const void * pSrc2, size_t nLength, Npp64f * pDst, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/** 
 * Device-buffer size (in bytes) for nppsAverageRelativeError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param nLength \ref length_specification.
 * \param hpBufferSize Required buffer size.  Important: 
 *        hpBufferSize is a <em>host pointer.</em>
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_SUCCESS
 */
NPP_EXTERN_C
NppStatus
nppsAverageRelativeErrorGetBufferSize_Ctx(NppDataType eSrcDstType, size_t nLength, size_t * hpBufferSize, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */


/** @} signal_statistical_functions */

#endif /* NV_NPPS_PLUS_STATISTICS_FUNCTIONS_H */
