 /* Copyright 2009-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPI_PLUS_STATISTICS_FUNCTIONS_H
#define NV_NPPI_PLUS_STATISTICS_FUNCTIONS_H
 
/**
 * \file nppiPlus_statistics_functions.h
 * NPP Image Processing Functionality.
 */

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 *  \page image_statistics_functions Statistical Operations
 *  @defgroup image_statistics_functions Statistical Operations
 *  @ingroup nppi
 *
 * Primitives for computing the statistical properties of an image.
 * Some statistical primitives also require scratch buffer during the computation. For details, please 
 * refer to \ref general_scratch_buffer.
 * 
 * @{
 *
 * These functions can be found in the nppist library. Linking to only the sub-libraries that you use can significantly
 * save link time, application load time, and CUDA runtime startup time when using dynamic libraries.
 *
 * \section CommonGetBufferHostSizeParameters
 * Common parameters for nppiGetBufferHostSize functions include:
 * \param oSizeROI \ref roi_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.  
 */

/** 
 * \section image_sum Sum
 * @defgroup image_sum Sum
 * Primitives for computing the sum of all the pixel values in an image.
 * @{
 */

/** @name Sum
 * @{
 * Given an image \f$pSrc\f$ with width \f$W\f$ and height \f$H\f$, the sum
 * will be computed as
 * \f[Sum = \sum_{j=0}^{H-1}\sum_{i=0}^{W-1}pSrc(j,i)\f]
 * All the results are stored in a 64-bit double precision format, 
 * except for two primitives nppiSum_8u64s_C1R_Ctx and nppiSum_8u64s_C4R_Ctx.
 * The sum functions require additional scratch buffer for computations. 
 *
 * \section CommonSumParameters Common parameters for nppiSum functions include:
 *
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
 *                      Use nppiSumGetBufferHostSize_XX_XXX to determine the minium number of bytes required.
 * \param pSum Pointer to the computed sum result.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */
 
#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image sum. 
 *
 * For common parameter descriptions, see \ref CommonSumParameters. 
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSum_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
            const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
            Npp64f * pSum, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);
/**
 * One-channel 8-bit unsigned image sum. The result is 64-bit long long integer.
 *
 * For common parameter descriptions, see \ref CommonSumParameters. 
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSumEx_8u64s_C1R_Ctx(const Npp8u * pSrc, int nSrcStep, NppiSize oSizeROI, Npp8u * pDeviceBuffer, Npp64s * pSum, NppStreamContext nppStreamCtx);

/**
 * Four-channel 8-bit unsigned image sum. The result is 64-bit long long integer.
 *
 * For common parameter descriptions, see \ref CommonSumParameters. 
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSumEx_8u64s_C4R_Ctx(const Npp8u * pSrc, int nSrcStep, NppiSize oSizeROI, Npp8u * pDeviceBuffer, Npp64s aSum[4], NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif

/** @} Sum */

/** @name SumGetBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the sum primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Buffer size for nppiSum.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcType source \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 *
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiSumGetBufferHostSize_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                             NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif

/** @} SumGetBufferHostSize */
/** @} image_sum */

/** 
 * \section image_min Min
 *  @defgroup image_min Min
 * Primitives for computing the minimal pixel value of an image.
 * @{
 */

/** @name Min
 * The scratch buffer is required by the min functions.
 *
 * \section CommonMinParameters Common parameters for nppiMin functions include:
 *
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
 *                      Use nppiMinGetBufferHostSize_XX_XXX to determine the minium number of bytes required.
 * \param pMin Pointer to the computed min of the same data type and channels as the source.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image min. 
 *
 * For common parameter descriptions, see \ref CommonMinParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMin_Ctx(NppDataType eSrcDstType, NppiChannels eSrcChannels,
            const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
            Npp8u * pDeviceBuffer, void * pMin, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} Min */

/** @name MinGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the min primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Buffer size for nppiMin.
 *
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 *
 */

NPP_EXTERN_C
NppStatus 
nppiMinGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                             NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MinGetBufferHostSize */
/** @} image_min */

/** 
 * \section image_min_index MinIndx
 * @defgroup image_min_index MinIndx
 * Primitives for computing the minimal value and its indices (X and Y coordinates) of an image.
 * @{
 */

/** @name MinIndx
 * If there are several minima in the selected ROI, the function returns one on the top leftmost position.
 * The scratch buffer is required by the functions.
 * 
 * \section CommonMinIndxParameters Common parameters for nppiMinIndx functions include:
 *
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMin Pointer to the computed min result of the same data type and channels as the source.
 * \param pIndexX Pointer to the X coordinate of the image min value.
 * \param pIndexY Ppointer to the Y coordinate of the image min value.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use nppiMinIndxGetBufferHostSize_XX_XXX to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image MinIndx.
 *
 * For common parameter descriptions, see \ref CommonMinIndxParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMinIndx_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                void * pMin, int * pIndexX, int * pIndexY, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} Min */

/** @name MinIndxGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the MinIndx primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Computes the dvice scratch buffer size (in bytes) for nppiMinIndx.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 *
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMinIndxGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                 NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MinIndxGetBufferHostSize */
/** @} image_min_index */

/** 
 * \section image_max Max
 * @defgroup image_max Max
 * Primitives for computing the maximal pixel value of an image.
 * 
 * \section CommonMaxParameters Common parameters for nppiMax functions include:
 *
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMax Pointer to the computed max of the same data type and channels as the source.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer.
 *                      Use nppiMaxGetBufferHostSize_XX_XXX to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 * 
 * @{
 */

/** @name Max
 *  The scratch buffer is required by the functions.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Max.
 *
 * For common parameter descriptions, see \ref CommonMaxParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMax_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
            const void * pSrc, int nSrcStep, NppiSize oSizeROI, void * pMax,
            Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MaxGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Max primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Buffer size for nppiMax.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 *
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMaxGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                             NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MaxGetBufferHostSize */
/** @} image_max */

/** 
 * \section image_max_index MaxIndx
 * @defgroup image_max_index MaxIndx
 * Primitives for computing the maximal value and its indices (X and Y coordinates) of an image.
 * @{
 */

/** @name MaxIndx
 * If there are several maxima in the selected region of interest, the function returns one on the top leftmost position. 
 * The scratch buffer is required by the functions.
 * 
 * \section CommonMaxIndxParameters Common parameters for nppiMaxIndx functions include:
 *
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMax Pointer to the computed max result of the same data type and channels as the source.
 * \param pIndexX Pointer to the X coordinate of the image max value.
 * \param pIndexY Ppointer to the Y coordinate of the image max value.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use nppiMaxIndxGetBufferHostSize_XX_XXX to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image MaxIndx.
 *
 * For common parameter descriptions, see \ref CommonMaxIndxParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMaxIndx_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                void * pMax, int * pIndexX, int * pIndexY, 
                Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MaxIndxGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the MaxIndx primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Computes the dvice scratch buffer size (in bytes) for nppiMaxIndx.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 *
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMaxIndxGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                 NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_max_index */

/** 
 * \section image_min_max MinMax
 * @defgroup image_min_max MinMax
 * Primitives for computing both the minimal and the maximal values of an image.
 * @{
 */

/** @name MinMax
 * The functions require the device scratch buffer.
 * 
 * \section CommonMinMaxParameters Common parameters for nppiMinMax functions include:
 * 
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type. 
 * \param eSrcChannels source number of channels, alpha is ignored for A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nCOI \ref channel_of_interest_number for multichannel C3 source.
 * \param pMin Pointer to the computed minimal result of the same data type and channels as the source.
 * \param pMax Pointer to the computed maximal result of the same data type and channels as the source.
 * \param pDeviceBuffer Buffer to a scratch memory. 
 *        Use nppiMinMax_XX_XXX to determine the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image MinMax.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiMinMax_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
               const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
               void * pMin, void * pMax, 
               Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Image MinMax_C3.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiMinMax_C3_Ctx(NppDataType eSrcType,
                  const void * pSrc, int nSrcStep, NppiSize oSizeROI, int nCOI,
                  void * pMin, void * pMax, 
                  Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MinMaxGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the MinMax primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiMinMax.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus
nppiMinMaxGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_min_max */

/** 
 * \section image_min_max_index MinMaxIndx
 * @defgroup image_min_max_index MinMaxIndx
 * Primitives for computing the minimal and the maximal values with their indices (X and Y coordinates) of an image.
 * @{
 */

/** @name MinMaxIndx
 * If there are several minima and maxima in the selected region of interest, the function returns ones on the top leftmost position.
 * The scratch buffer is required by the functions.
 * 
 * \section CommonMinMaxIndxParameters Common parameters for nppiMinIndx functions include:
 * 
 * Supported source data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcType source \ref image_data_type.
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.  
 * \param nCOI \ref channel_of_interest_number.
 * \param pMinValue Pointer to the minimum value of the same data type and channels as the source.
 * \param pMaxValue Pointer to the maximum value of the same data type and channels as the source.
 * \param pMinIndex Pointer to the indicies (X and Y coordinates) of the minimum value.
 * \param pMaxIndex Pointer to the indicies (X and Y coordinates) of the maximum value.
 * \param pDeviceBuffer Buffer to a scratch memory. Use nppiMinMaxIndxGetBufferHostSize_XX_XXX to determine
 *          the minium number of bytes required.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * If any of pMinValue, pMaxValue, pMinIndex, or pMaxIndex is not needed, zero pointer must be passed correspondingly.
 * If the mask is filled with zeros, then all the returned values are zeros, i.e., pMinIndex = {0, 0}, pMaxIndex = {0, 0},
 * pMinValue = 0, pMaxValue = 0.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Computes the minimal and the maximal pixel values with their X and Y coordinates of image. 
 *
 * For common parameter descriptions, see \ref CommonMinMaxIndxParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMinMaxIndx_C1_Ctx(NppDataType eSrcType,
                      const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                      void * pMinValue, void * pMaxValue, NppiPoint * pMinIndex, NppiPoint * pMaxIndex, 
                      Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Computes the minimal and the maximal pixel values with their X and Y coordinates of image. 
 *
 * For common parameter descriptions, see \ref CommonMinMaxIndxParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMinMaxIndx_C3_Ctx(NppDataType eSrcType,
                      const void * pSrc, int nSrcStep, NppiSize oSizeROI, int nCOI,
                      void * pMinValue, void * pMaxValue, NppiPoint * pMinIndex, NppiPoint * pMaxIndex, 
                      Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Masked image MinMaxIndx_C1M.
 *
 * For common parameter descriptions, see \ref CommonMinMaxIndxParameters.
 *  
 */
NPP_EXTERN_C
NppStatus 
nppiMinMaxIndx_C1M_Ctx(NppDataType eSrcType,
                       const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                       void * pMinValue, void * pMaxValue, NppiPoint * pMinIndex, NppiPoint * pMaxIndex, 
                       Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Masked image MinMaxIndx_C3M.
 *
 * For common parameter descriptions, see \ref CommonMinMaxIndxParameters.
 *  
 */
NPP_EXTERN_C
NppStatus
nppiMinMaxIndx_C3M_Ctx(NppDataType eSrcType,
                       const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, int nCOI,
                       void * pMinValue, void * pMaxValue, NppiPoint * pMinIndex, NppiPoint * pMaxIndex, 
                       Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MinMaxIndxGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the MinMaxIndx primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiMinMaxIndx.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMinMaxIndxGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_min_max_index */

/** 
 * \section image_mean Mean
 * @defgroup image_mean Mean
 * Primitives for computing the arithmetic mean of all the pixel values in an image.
 * @{
 */

/** @name Mean
 * Given an image \f$pSrc\f$ with width \f$W\f$ and height \f$H\f$, the arithmetic mean
 * will be computed as
 * \f[Mean = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}pSrc(j,i)\f]
 * The mean functions require additional scratch buffer for computations.
 * 
 * \section CommonMeanParameters Common parameters for nppiMean functions include:
 * 
 * Supported source data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification. 
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param nCOI \ref channel_of_interest_number.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use nppiMeanGetBufferHostSize_XX_XXX to determine the minium number of bytes required.
 * \param pMean Pointer to the computed mean result.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.s
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Mean.
 *
 * For common parameter descriptions, see \ref CommonMeanParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMean_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
             const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
             Npp8u * pDeviceBuffer, Npp64f * pMean, 
             NppStreamContext nppStreamCtx);

/**
 * Masked image Mean.
 *
 * For common parameter descriptions, see \ref CommonMeanParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMean_C1M_Ctx(NppDataType eSrcType,
                 const void * pSrc, int nSrcStep, const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                 Npp8u * pDeviceBuffer, Npp64f * pMean, 
                 NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image Mean affecting only single channel.
 *
 * For common parameter descriptions, see \ref CommonMeanParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMean_C3M_Ctx(NppDataType eSrcType,
                 const void * pSrc, int nSrcStep, const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, int nCOI, 
                 Npp8u * pDeviceBuffer, Npp64f * pMean, 
                 NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MeanGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Mean primitives.
 *  
 * \section CommonMeanGetBufferHostSizeParameters Common parameters for nppiMeanGetBufferHostSize functions include:
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes.  
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Buffer size for nppiMean. 
 *  
 * For common parameter descriptions, see \ref CommonMeanGetBufferHostSizeParameters.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiMeanGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                              NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

/**
 * Buffer size for nppiMean with mask. 
 *  
 * For common parameter descriptions, see \ref CommonMeanGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMaskedMeanGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_mean */

/** 
 * \section image_mean_stddev Mean_StdDev
 * @defgroup image_mean_stddev Mean_StdDev
 * Primitives for computing both the arithmetic mean and the standard deviation of an image.
 * @{
 */

/** @name Mean_StdDev
 * Given an image \f$pSrc\f$ with width \f$W\f$ and height \f$H\f$, the mean and the standard deviation
 * will be computed as
 * \f[Mean = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}pSrc(j,i)\f]
 * \f[StdDev = \sqrt{\frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}(pSrc(j,i)-Mean)^2}\f]
 * The Mean_StdDev primitives require additional scratch buffer for computations.
 *  
 * \section CommonMeanStdDevParameters Common parameters for nppiMean_StdDev functions include:
 *
 * Supported source data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param nCOI \ref channel_of_interest_number.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 *        Use nppiMeanStdDevGetBufferHostSize_8u_C1R to determine the minium number of bytes required.
 * \param pMean Pointer to the computed mean.
 * \param pStdDev Pointer to the computed standard deviation.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified.
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Mean_StdDev.
 * 
 * For common parameter descriptions, see \ref CommonMeanStdDevParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMean_StdDev_C1_Ctx(NppDataType eSrcType,
                       const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                       Npp8u * pDeviceBuffer, Npp64f * pMean, Npp64f * pStdDev,
                       NppStreamContext nppStreamCtx);

/**
 * Single channel masked image Mean_StdDev.
 * 
 * For common parameter descriptions, see \ref CommonMeanStdDevParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMean_StdDev_C1M_Ctx(NppDataType eSrcType,
                        const void * pSrc, int nSrcStep, const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                        Npp8u * pDeviceBuffer, Npp64f * pMean, Npp64f * pStdDev,
                        NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

 /** @name Channel Mean_StdDev
 * See \ref channel_of_interest.
 * @{
 */
 
#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Three-channel Mean_StdDev affecting only single channel.
 * 
 * For common parameter descriptions, see \ref CommonMeanStdDevParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMean_StdDev_C3_Ctx(NppDataType eSrcType,
                       const void * pSrc, int nSrcStep, NppiSize oSizeROI, int nCOI,
                       Npp8u * pDeviceBuffer, Npp64f * pMean, Npp64f * pStdDev,
                       NppStreamContext nppStreamCtx);
/**
 * Masked three-channel 8-bit unsigned image Mean_StdDev.
 * 
 * For common parameter descriptions, see \ref CommonMeanStdDevParameters.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiMean_StdDev_C3M_Ctx(NppDataType eSrcType,
                        const void * pSrc, int nSrcStep, const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, int nCOI,
                        Npp8u * pDeviceBuffer, Npp64f * pMean, Npp64f * pStdDev,
                        NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MeanStdDevGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Mean_StdDev primitives.
 *  
 * \section CommonMeanStdDevGetBufferHostSizeParameters Common parameters for nppiMeanStdDevGetBufferHostSize functions include:
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiMean_StdDev.
 *  
 *
 * For common parameter descriptions, see \ref CommonMeanStdDevGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMeanStdDevGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * Buffer size for nppiMaskedMean_StdDev.
 *  
 *
 * For common parameter descriptions, see \ref CommonMeanStdDevGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiMaskedMeanStdDevGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                          NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_mean_stddev */

/** 
 * \section image_norm Image Norms
 * @defgroup image_norm Image Norms
 *
 * Primitives for computing the norms of an image, the norms of difference, and the relative errors of two images.
 * Given an image \f$pSrc\f$ with width \f$W\f$ and height \f$H\f$, 
 * -# The infinity norm (Norm_Inf) is defined as the largest absolute pixel value of the image.
 * -# The L1 norm (Norm_L1) is defined as the sum of the absolute pixel value of the image, i.e., \f[Norm\_L1 = \sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left| pSrc(j,i)\right|\f].
 * -# The L2 norm (Norm_L2) is defined as the square root of the sum of the squared absolute pixel value of the image, i.e.,\f[Norm\_L2 = \sqrt{\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left| pSrc(j,i)\right| ^2}\f].
 * 
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * -# The infinity norm of differece (NormDiff_Inf) is defined as the largest absolute difference between pixels of two images.
 * -# The L1 norm of differece (NormDiff_L1) is defined as the sum of the absolute difference between pixels of two images, i.e., \f[NormDiff\_L1 = \sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left| pSrc1(j,i)-pSrc2(j,i)\right|\f].
 * -# The L2 norm of differece (NormDiff_L2) is defined as the squared root of the sum of the squared absolute difference between pixels of two images, i.e.,\f[NormDiff\_L2 = \sqrt{\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left| pSrc1(j,i)-pSrc2(j,i)\right| ^2}\f].
 * 
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * -# The relative error for the infinity norm of differece (NormRel_Inf) is defined as NormDiff_Inf divided by the infinity norm of the second image, i.e., \f[NormRel\_Inf = \frac{NormDiff\_Inf}{Norm\_Inf_{src2}}\f]
 * -# The relative error for the L1 norm of differece (NormRel_L1) is defined as NormDiff_L1 divided by the L1 norm of the second image, i.e., \f[NormRel\_L1 = \frac{NormDiff\_L1}{Norm\_L1_{src2}}\f]
 * -# The relative error for the L2 norm of differece (NormRel_L2) is defined as NormDiff_L2 divided by the L2 norm of the second image, i.e., \f[NormRel\_L2 = \frac{NormDiff\_L2}{Norm\_L2_{src2}}\f]
 * 
 * The norm functions require the addition device scratch buffer for the computations.
 * 
 * \section CommonNormParameters Common parameters for nppiNorm functions include:
 *
 * Supported source data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMask \ref mask_image_pointer.
 * \param nMaskStep \ref mask_image_line_step.
 * \param nCOI \ref channel_of_interest_number.
 * \param pNorm Pointer to the norm value of same data type and channels as source.
 * \param pNormDiff Pointer to the computed norm of differences of same number of channels. 
 * \param pNormRel Pointer to the computed relative error for the infinity norm of two images.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 *        Use nppiNormInfGetBufferHostSize to compute the required size (in bytes).
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_COI_ERROR if an invalid channel of interest is specified, or NPP_NOT_EVEN_STEP_ERROR if an invalid floating-point image is specified.
 * 
 * @{
 *
 */ 

/** 
 * \section image_inf_norm Norm_Inf
 * @defgroup image_inf_norm Norm_Inf
 * Primitives for computing the infinity norm of an image. 
 * @{
 */

/** @name Basic Norm_Inf
 *   
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Norm_Inf.
 *
 * For common parameter descriptions, see \ref CommonNormParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiNorm_Inf_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                 const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                 Npp64f * pNorm, Npp8u * pDeviceBuffer,
                 NppStreamContext nppStreamCtx);

/**
 * Masked single channel image Norm_Inf.
 *
 * For common parameter descriptions, see \ref CommonNormParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNorm_Inf_C1M_Ctx(NppDataType eSrcType,
                     const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                     Npp64f * pNorm, Npp8u * pDeviceBuffer,
                     NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image Norm_Inf affecting only single channel.
 *
 * For common parameter descriptions, see \ref CommonNormParameters.
 */
NPP_EXTERN_C
NppStatus
nppiNorm_Inf_C3M_Ctx(NppDataType eSrcType,
                     const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                     int nCOI, Npp64f * pNorm, Npp8u * pDeviceBuffer,
                     NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name NormInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Norm_Inf primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** Buffer size for nppiNorm_Inf.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * 
 * 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiNormInfGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                 NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_inf_norm */


/** 
 * \section image_L1_norm Norm_L1
 * @defgroup image_L1_norm Norm_L1
 * Primitives for computing the L1 norm of an image. 
 * @{
 */

/** @name Basic Norm_L1
 * @{
 */ 

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Norm_L1.
 * 
 * For common parameter descriptions, see \ref CommonNormParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiNorm_L1_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                Npp64f * pNorm, Npp8u * pDeviceBuffer,
                NppStreamContext nppStreamCtx);

/**
 * Masked one channel image Norm_L1.
 * 
 * For common parameter descriptions, see \ref CommonNormParameters.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiNorm_L1_C1M_Ctx(NppDataType eSrcType,
                    const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                    Npp64f * pNorm, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

/**
 * Masked three-channel Norm_L1 affecting only single channel.
 *
 * For common parameter descriptions, see \ref CommonNormParameters.
 * 
 */
NPP_EXTERN_C
NppStatus
nppiNorm_L1_C3M_Ctx(NppDataType eSrcType,
                    const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                    int nCOI, Npp64f * pNorm, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
 
 /** @name NormL1GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Norm_L1 primitives.
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiNorm_L1.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 *
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 *
 */
NPP_EXTERN_C
NppStatus 
nppiNormL1GetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @}  */
/** @} image_L1_norm */

/** 
 * \section image_L2_norm Norm_L2
 * @defgroup image_L2_norm Norm_L2
 * Primitives for computing the L2 norm of an image. 
 * @{
 */

/** @name Basic Norm_L2
 * Computes the L2 norm of an image. 
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Norm_L2.
 *
 * For common parameter descriptions, see \ref CommonNormParameters.
 * 
 */
NPP_EXTERN_C
NppStatus 
nppiNorm_L2_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                Npp64f * pNorm, Npp8u * pDeviceBuffer,
                NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image Norm_L2.
 * For common parameter descriptions, see \ref CommonNormParameters.  
 */
NPP_EXTERN_C
NppStatus 
nppiNorm_L2_C1M_Ctx(NppDataType eSrcType,
                    const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                    Npp64f * pNorm, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image Norm_L2 affecting only a single channel.
 */
NPP_EXTERN_C
NppStatus
nppiNorm_L2_C3M_Ctx(NppDataType eSrcType,
                    const void * pSrc, int nSrcStep,  const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                    int nCOI, Npp64f * pNorm, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name NormL2GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Norm_L2 primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiNorm_L2.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormL2GetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_L2_norm */

/** 
 * \section image_inf_normdiff NormDiff_Inf
 * @defgroup image_inf_normdiff NormDiff_Inf
 * Primitives for computing the infinity norm of difference of pixels between two images.
 * @{
 */

/** @name Basic NormDiff_Inf
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image NormDiff_Inf.
 * For common parameter descriptions, see \ref CommonNormParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormDiff_Inf_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                     const void * pSrc1, int nSrc1Step, 
                     const void * pSrc2, int nSrc2Step, NppiSize oSizeROI, 
                     Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                     NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image NormDiff_Inf.
 * For common parameter descriptions, see \ref CommonNormParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormDiff_Inf_C1M_Ctx(NppDataType eSrcType,
                         const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                         const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                         Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                         NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image NormDiff_Inf affecting only single channel.
 * For common parameter descriptions, see \ref CommonNormParameters.
 */
NPP_EXTERN_C
NppStatus
nppiNormDiff_Inf_C3M_Ctx(NppDataType eSrcType,
                         const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                         const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                         int nCOI, Npp64f * pNorm, Npp8u * pDeviceBuffer,
                         NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
 
/** @name NormDiffInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiNormDiff_Inf.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormDiffInfGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                     NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_inf_normdiff */

/** 
 * \section image_L1_normdiff NormDiff_L1
 * @defgroup image_L1_normdiff NormDiff_L1
 * Primitives for computing the L1 norm of difference of pixels between two images.
 * @{
 */

/** @name Basic NormDiff_L1
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image NormDiff_L1.
 * For common parameter descriptions, see \ref CommonNormParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormDiff_L1_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                    const void * pSrc1, int nSrc1Step, 
                    const void * pSrc2, int nSrc2Step, NppiSize oSizeROI, 
                    Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image NormDiff_L1.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormDiff_L1_C1M_Ctx(NppDataType eSrcType,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                        const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                        Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                        NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image NormDiff_L1 affecting only single channel.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormDiff_L1_C3M_Ctx(NppDataType eSrcType,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                        const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, int nCOI, 
                        Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                        NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
 
 /** @name NormDiffL1GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_L1 primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L1.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormDiffL1GetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_L1_normdiff */

/** 
 * \section image_L2_normdiff NormDiff_L2
 * @defgroup image_L2_normdiff NormDiff_L2
 * Primitives for computing the L2 norm of difference of pixels between two images.
 * @{
 */

/** @name Basic NormDiff_L2
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image NormDiff_L2.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormDiff_L2_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                    const void * pSrc1, int nSrc1Step, 
                    const void * pSrc2, int nSrc2Step, NppiSize oSizeROI, 
                    Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image NormDiff_L2.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormDiff_L2_C1M_Ctx(NppDataType eSrcType,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                        const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                        Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                        NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image NormDiff_L2 affecting only single channel.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormDiff_L2_C3M_Ctx(NppDataType eSrcType,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                        const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, int nCOI, 
                        Npp64f * pNormDiff, Npp8u * pDeviceBuffer,
                        NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name NormDiffL2GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_L2 primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Computes the device scratch buffer size (in bytes) for nppiNormDiff_L2.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormDiffL2GetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_L2_normdiff */

/** 
 * \section image_inf_normrel NormRel_Inf
 * @defgroup image_inf_normrel NormRel_Inf
 * Primitives for computing the relative error of infinity norm between two images.
 * @{
 */

/** @name Basic NormRel_Inf
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image NormRel_Inf.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_Inf_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                    const void * pSrc1, int nSrc1Step, 
                    const void * pSrc2, int nSrc2Step, NppiSize oSizeROI, 
                    Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                    NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image NormRel_Inf.
 * For common parameter descriptions, see \ref CommonNormParameters. 
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_Inf_C1M_Ctx(NppDataType eSrcType,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                        const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                        Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                        NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image NormRel_Inf affecting only signle channel.
 * For common parameter descriptions, see \ref CommonNormParameters. 
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_Inf_C3M_Ctx(NppDataType eSrcType,
                        const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                        const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                        int nCOI, Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                        NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name NormRelInfGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormRel_Inf primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_Inf.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormRelInfGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @} image_inf_normrel */

/** 
 * \section image_L1_normrel NormRel_L1
 * @defgroup image_L1_normrel NormRel_L1
 * Primitives for computing the relative error of L1 norm between two images.
 * @{
 */

/** @name Basic NormRel_L1
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image NormRel_L1.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_L1_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                   const void * pSrc1, int nSrc1Step, 
                   const void * pSrc2, int nSrc2Step, NppiSize oSizeROI, 
                   Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                   NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image NormRel_L1.
 * For common parameter descriptions, see \ref CommonNormParameters. 
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_L1_C1M_Ctx(NppDataType eSrcType,
                      const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                      const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                      Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                      NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image NormRel_L1 affecting only signle channel.
 * For common parameter descriptions, see \ref CommonNormParameters. 
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_L1_C3M_Ctx(NppDataType eSrcType,
                       const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                       const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                       int nCOI, Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                       NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name NormRelL1GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormRel_L1 primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L1.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormRelL1GetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                   NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @} image_L1_normrel */

/** 
 * \section image_L2_normrel NormRel_L2
 * @defgroup image_L2_normrel NormRel_L2
 * Primitives for computing the relative error of L2 norm between two images.
 * @{
 */

/** @name Basic NormRel_L2
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * image NormRel_L2.
 * For common parameter descriptions, see \ref CommonNormParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_L2_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                   const void * pSrc1, int nSrc1Step, 
                   const void * pSrc2, int nSrc2Step, NppiSize oSizeROI, 
                   Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                   NppStreamContext nppStreamCtx);

/**
 * Masked one-channel image NormRel_L2.
 * For common parameter descriptions, see \ref CommonNormParameters. 
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_L2_C1M_Ctx(NppDataType eSrcType,
                       const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                       const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                       Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                       NppStreamContext nppStreamCtx);

/**
 * Masked three-channel image NormRel_L2 affecting only signle channel.
 * For common parameter descriptions, see \ref CommonNormParameters. 
*/
NPP_EXTERN_C
NppStatus 
nppiNormRel_L2_C3M_Ctx(NppDataType eSrcType,
                       const void * pSrc1, int nSrc1Step, const void * pSrc2, int nSrc2Step,  
                       const Npp8u * pMask, int nMaskStep, NppiSize oSizeROI, 
                       int nCOI, Npp64f * pNormRel, Npp8u * pDeviceBuffer,
                       NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name NormRelL2GetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormRel_L2 primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Computes the device scratch buffer size (in bytes) for nppiNormRel_L2.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiNormRelL2GetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                   NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_L2_normrel */
/** @} image_norm */

/** 
 * \section image_dot_prod DotProd
 * @defgroup image_dot_prod DotProd
 * Primitives for computing the dot product of two images.
 * @{
 *
 */

/** @name DotProd
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the dot product will be computed as
 * \f[DotProd = \sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[pSrc1(j,i)\cdot pSrc2(j,i)]\f]
 * The functions require additional scratch buffer for computations.
 * \section CommonDotProdParameters Common parameters for nppiDotProd functions include:
 * Supported source data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pDotProd Pointer to the computed dot product of the two images of the same number of channels. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image DotProd.
 * For common parameter descriptions, see \ref CommonDotProdParameters.
*/
NPP_EXTERN_C
NppStatus 
nppiDotProd_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                  const void * pSrc1, int nSrc1Step, 
                  const void * pSrc2, int nSrc2Step, NppiSize oSrc1SizeROI, 
                  Npp64f * pDotProd, Npp8u * pDeviceBuffer,
                  NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name DotProdGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the Mean_StdDev primitives.
 * @{
 */ 

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppiDotProd.
 *  
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiDotProdGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                 NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_dot_prod */

/** 
 * \section image_count_in_range CountInRange.
 * @defgroup image_count_in_range CountInRange.
 * Primitives for computing the amount of pixels that fall into the specified intensity range.
 * @{
 *
 */

/** @name CountInRange
 * The lower bound and the upper bound are inclusive. The functions require additional
 * scratch buffer for computations. 
 * \section CommonCountInRangeParameters Common parameters for nppiCountInRange functions include:
 * Supported source data types include NPP_8U, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pCounts Pointer to the number of pixels that fall into the specified range of the same number of channels as the source.
 * \param pLowerBound Pointer to the lower bound of the specified range of the same data type and number of channels as the source.
 * \param pUpperBound Pointer to the upper bound of the specified range of the same data type and number of channels as the source.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_RANGE_ERROR if the lower bound
 * is larger than the upper bound.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CountInRange.
 * For common parameter descriptions, see \ref CommonCountInRangeParameters. 
 */
NPP_EXTERN_C
NppStatus 
nppiCountInRange_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                     const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                     int * pCounts, void * pLowerBound, void * pUpperBound, 
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
 
 /** @name CountInRangeGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the CountInRange primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Device scratch buffer size (in bytes) for nppiCountInRange.
 *  
 * Supported data types include NPP_8U, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiCountInRangeGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                      NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_count_in_range */

/** 
 * \section image_maxevery MaxEvery
 * @defgroup image_maxevery MaxEvery
 * Primitives for computing the maximal value of the pixel pair from two images.
 * @{
 *
 */

/** @name MaxEvery
 * The maximum is stored into the second image.
 * \section CommonMaxEveryParameters Common parameters for nppiMaxEvery functions include: 
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * In place image MaxEvery.
 * For common parameter descriptions, see \ref CommonMaxEveryParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMaxEvery_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                   const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_maxevery*/

/** 
 * \section image_minevery MinEvery
 * @defgroup image_minevery MinEvery
 * Primitives for computing the minimal value of the pixel pair from two images.
 * @{
 *
 */

/** @name MinEvery
 * The minimum is stored into the second image.
 * \section CommonMinEveryParameters Common parameters for nppiMinEvery functions include:
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcDstType source \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSrcDst \ref in_place_image_pointer.
 * \param nSrcDstStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * In place image MinEvery.
 * For common parameter descriptions, see \ref CommonMinEveryParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMinEvery_I_Ctx(NppDataType eSrcDstType, NppiChannels eSrcDstChannels,
                   const void * pSrc, int nSrcStep, void * pSrcDst, int nSrcDstStep, 
                   NppiSize oSizeROI, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_minevery */

/** 
 * \section image_integral Integral
 * @defgroup image_integral Integral
 * Primitives for computing the integral image of a given image.
 * @{
 *
 */
                                  
/** @name Integral
 * Given an input image \f$pSrc\f$ and the specified value \f$nVal\f$, 
 * the pixel value of the integral image \f$pDst\f$ at coordinate 
 * (i, j) will be computed as
 * \f[pDst(j,i) = nVal + \sum_{l=0}^{j-1}\sum_{k=0}^{i-1}pSrc(l,k)\f]
 * If the size of the input image is \f$W \times H\f$, the size of 
 * the integral image will be \f$(W+1) \times (H+1)\f$.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One-channel 8-bit unsigned image Integral with 32-bit signed output.
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oROI \ref roi_specification.
 * \param nVal The value to add to pDst image pixels
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiIntegral_8u32s_C1R_Ctx(const Npp8u  * pSrc, int nSrcStep, 
                                 Npp32s * pDst, int nDstStep, 
                           NppiSize oROI, Npp32s nVal, NppStreamContext nppStreamCtx);

/**
 * One-channel 8-bit unsigned image Integral with 32-bit floating point output.
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oROI \ref roi_specification.
 * \param nVal The value to add to pDst image pixels
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiIntegral_8u32f_C1R_Ctx(const Npp8u  * pSrc, int nSrcStep, 
                                 Npp32f * pDst, int nDstStep, 
                           NppiSize oROI, Npp32f nVal, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} Integral */

/** 
 * \section image_sqrintegral SqrIntegral
 * @defgroup image_sqrintegral SqrIntegral
 * Primitives for computing both the integral and the squared integral images of a given image.
 * @{
 *
 */

/** @name SqrIntegral
 *
 * Given an input image \f$pSrc\f$ and the specified value \f$nVal\f$, 
 * the pixel value of the integral image \f$pDst\f$ at coordinate 
 * (i, j) will be computed as
 * \f[pDst(j,i) = nVal + \sum_{l=0}^{j-1}\sum_{k=0}^{i-1}pSrc(l,k)\f]
 * Given an input image \f$pSrc\f$ and the specified value \f$nValSqr\f$, 
 * the pixel value of the squared integral image \f$pSqr\f$ at coordinate 
 * (i, j) will be computed as
 * \f[pSqr(j,i) = nValSqr + \sum_{l=0}^{j-1}\sum_{k=0}^{i-1}{pSrc(l,k)}^2\f]
 * If the size of the input image is \f$W \times H\f$, the size of 
 * the squared integral image will be \f$(W+1) \times (H+1)\f$.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One-channel 8-bit unsigned image SqrIntegral. 
 * Destination integral image and square integral image are 32-bit signed int.
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param pSqr \ref destination_image_pointer.
 * \param nSqrStep \ref destination_image_line_step.
 * \param oSrcROI \ref roi_specification.
 * \param nVal The value to add to pDst image pixels
 * \param nValSqr The value to add to pSqr image pixels
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSqrIntegral_8u32s_C1R_Ctx(const Npp8u  * pSrc, int nSrcStep, 
                                    Npp32s * pDst, int nDstStep, 
                                    Npp32s * pSqr, int nSqrStep,
                              NppiSize oSrcROI, Npp32s nVal, Npp32s nValSqr, NppStreamContext nppStreamCtx);

/**
 * One-channel 8-bit unsigned image SqrIntegral. 
 * Destination integral image is 32-bit signed int. 
 * Destination square integral image is 64-bit double floating point.
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param pSqr \ref destination_image_pointer.
 * \param nSqrStep \ref destination_image_line_step.
 * \param oSrcROI \ref roi_specification.
 * \param nVal The value to add to pDst image pixels
 * \param nValSqr The value to add to pSqr image pixels
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSqrIntegral_8u32s64f_C1R_Ctx(const Npp8u  * pSrc, int nSrcStep, 
                                       Npp32s * pDst, int nDstStep, 
                                       Npp64f * pSqr, int nSqrStep,
                                 NppiSize oSrcROI, Npp32s nVal, Npp64f nValSqr, NppStreamContext nppStreamCtx);

/**
 * One-channel 8-bit unsigned image SqrIntegral. 
 * Destination integral image is 32-bit floating point.
 * Destination square integral image is 64-bit double floating point.
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param pSqr \ref destination_image_pointer.
 * \param nSqrStep \ref destination_image_line_step.
 * \param oSrcROI \ref roi_specification.
 * \param nVal The value to add to pDst image pixels
 * \param nValSqr The value to add to pSqr image pixels
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSqrIntegral_8u32f64f_C1R_Ctx(const Npp8u  * pSrc, int nSrcStep, 
                                       Npp32f * pDst, int nDstStep, 
                                       Npp64f * pSqr, int nSqrStep,
                                 NppiSize oSrcROI, Npp32f nVal, Npp64f nValSqr, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @}  */
/** @} image_sqrintegral */

/** 
 * \section image_rectstddev RectStdDev
 * @defgroup image_rectstddev RectStdDev
 * Primitives for computing the stansdard deviation of the integral images.
 * The function computes the standard deviation of the pixel in the rectangular window with the 
 * integral image \f$pSrc\f$ and the squared integral image \f$pSqr\f$, which can be obtained by calling
 * \ref image_integral and \ref image_sqrintegral.
 * 
 * The standard deviation of the pixel \f$(j, i)\f$ can be computed using the formula:
 * \f[pDst(j, i) = \sqrt{max(0, \frac{\sum(SqrIntegral)\cdot N - (\sum(Integral))^2}{N^2})}\f]
 * where \f$\sum(SqrIntegral) = pSqr[j+oRect.y+oRect.height, i+oRect.x+oRect.width] - pSqr[j+oRect.y,i+oRect.x+oRect.width]
 * - pSqr[j+oRect.y+oRect.height, i+oRect.x] + pSqr[j+oRect.y, i+oRect.x]\f$,
 * \f$\sum(Integral) = pSrc[j+oRect.y+oRect.height, i+oRect.x+oRect.width] - pSrc[j+oRect.y,i+oRect.x+oRect.width]
 * - pSrc[j+oRect.y+oRect.height, i+oRect.x] + pSrc[j+oRect.y, i+oRect.x]\f$,
 * \f$N = oRect.width \cdot oRect.height\f$.
 *
 * The size of the \f$pSrc\f$ and \f$pSqr\f$ should be \f$(oSizeROI.width + oRect.x + oRect.width, 
 * oSizeROI.height + oRect.y + oRect.height).\f$
 * @{
 *
 */

/** @name RectStdDev
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One-channel 32-bit floating point image RectStdDev.
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSqr \ref destination_image_pointer.
 * \param nSqrStep \ref destination_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param oRect rectangular window
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiRectStdDev_32f_C1R_Ctx(const Npp32f *pSrc, int nSrcStep, const Npp64f *pSqr, int nSqrStep, 
                                 Npp32f *pDst, int nDstStep, NppiSize oSizeROI, NppiRect oRect, NppStreamContext nppStreamCtx);

/**
 * One-channel 32-bit signed image RectStdDev, scaled by \f$2^(-nScaleFactor)\f$. 
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSqr \ref destination_image_pointer.
 * \param nSqrStep \ref destination_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param oRect rectangular window
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiRectStdDev_32s_C1RSfs_Ctx(const Npp32s *pSrc, int nSrcStep, const Npp32s *pSqr, int nSqrStep, 
                                    Npp32s *pDst, int nDstStep, NppiSize oSizeROI, NppiRect oRect, int nScaleFactor, NppStreamContext nppStreamCtx);

 /**
 * One-channel 32-bit signed image RectStdDev. 
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param pSqr \ref destination_image_pointer.
 * \param nSqrStep \ref destination_image_line_step.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param oRect rectangular window
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiRectStdDev_32s32f_C1R_Ctx(const Npp32s *pSrc, int nSrcStep, const Npp64f *pSqr, int nSqrStep, 
                                    Npp32f *pDst, int nDstStep, NppiSize oSizeROI, NppiRect oRect, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @}  */
/** @} image_rectstddev */

/** 
 * \section image_histogrameven HistogramEven
 * @defgroup image_histogrameven HistogramEven
 * Primitives for computing the histogram of an image with evenly distributed bins.
 * @{
 *
 */

/** @name HistogramEven
 * The \f$nLowerLevel\f$ (inclusive) and \f$nUpperLevel\f$ (exclusive) define the boundaries of the range, 
 * which are evenly segmented into \f$nLevel - 1\f$ bins. The computed histogram is stored in \f$pHist\f$. 
 * The levels are calculated by another primitive nppiEvenLevelsHost_32s and are stored 
 * in a host pointer \f$hpLevels\f$. The number of levels is also \f$nLevel - 1\f$.
 * The histogram \f$pHist[k]\f$ is defined as the total number of pixels that fall into the range:
 * \f$hpLevels[k] <= pSrc(j, i) < hpLevels[k+1]\f$.
 * The functions require additional scratch buffer for computations. 
 * 
 * \section CommonHistogramEvenParameters Common parameters for nppiHistogramEven functions include: 
 *  
 * Supported source data types include NPP_8U, NPP_16U, NPP_16S. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pHist Pointer to array of at least pLevels-1 of eSrcDstChannels per level that receives the computed histogram. 
 * \param pLevels Pointer to array of nLevelCounts of eSrcDstChannels.
 * \param pLowerLevel Pointer to lower boundary of lowest level bin of eSrcDstChannels.
 * \param pUpperLevel Pointer to upper boundary of highest level bin of eSrcDstChannels.
 * \param pBuffer Pointer to appropriately sized scratch buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Compute levels with even distribution.
 *
 * \param hpLevels A host pointer to array which receives the levels being
 *        computed. The array needs to be of size nLevels. 
 * \param nLevels The number of levels being computed. nLevels must be at least 2.
 * \param nLowerLevel Lower boundary value of the lowest level.
 * \param nUpperLevel Upper boundary value of the greatest level.
 * \return image_data_error_codes, or NPP_HISTO_NUMBER_OF_LEVELS_ERROR if an invalid nLevels is specified.
 */
NPP_EXTERN_C
NppStatus
nppiEvenLevelsHost_32s(Npp32s * hpLevels, int nLevels, Npp32s nLowerLevel, Npp32s nUpperLevel);

/**
 * Image HistogramEven.
 * For common parameter descriptions, see \ref CommonHistogramEvenParameters. 
 */
NPP_EXTERN_C
NppStatus 
nppiHistogramEven_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                      const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                      Npp32s ** pHist, int * pLevels, Npp32s * pLowerLevels, Npp32s * pUpperLevels, 
                      Npp8u * pBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
 
/** @name HistogramEvenGetBufferSize
 * Companion primitives for computing the device buffer size (in bytes) required by the HistogramEven primitives.
 * \section CommonHistogramEvenGetBufferSizeParameters Common parameters for nppiHistogramEvenGetBufferSize functions include: 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels source image number of channels.
 * \param oSizeROI \ref roi_specification.
 * \param pLevels Pointer to array of number of levels in the histogram for each eSrcChannels.
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Buffer size for nppiHistogramEven.
 * For common parameter descriptions, see \ref CommonHistogramEvenGetBufferSizeParameters. 
 */
NPP_EXTERN_C
NppStatus 
nppiHistogramEvenGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                       NppiSize oSizeROI, int * pLevels, size_t * hpBufferSize /* host pointer */, 
                                       NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @} image_histogrameven */


/** 
 * \section image_histogramrange HistogramRange
 * @defgroup image_histogramrange HistogramRange
 * Primitives for computing the histogram of an image within specified ranges.
 * @{
 *
 */

/** @name HistogramRange
 * The histogram is computed according to the ranges provided in \f$pLevels\f$. 
 * The histogram \f$pHist[k]\f$ is defined as the total number of pixels that fall into the range:
 * \f$pLevels[k] <= pSrc(j, i) < pLevels[k+1]\f$. The number of the histogram bins is \f$nLevel - 1\f$.
 * The functions require additional scratch buffer for computations.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image HistogramRange.
 * For common parameter descriptions, see \ref CommonHistogramEvenParameters. 
 */ 
NPP_EXTERN_C
NppStatus 
nppiHistogramRange_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                       const void * pSrc, int nSrcStep, NppiSize oSizeROI, 
                       Npp32s ** pHist, Npp32s ** pLevels, int * pLevelsCount, 
                       Npp8u * pBuffer, NppStreamContext nppStreamCtx);

/**
 * 32-bit floating point HistogramRange_32f.
 * For common parameter descriptions, see \ref CommonHistogramEvenParameters. 
 */ 
NPP_EXTERN_C
NppStatus 
nppiHistogramRange_32f_Ctx(NppiChannels eSrcChannels,
                           const Npp32f * pSrc, int nSrcStep, NppiSize oSizeROI, 
                           Npp32s ** pHist, Npp32f ** pLevels, int * pLevelsCount, 
                           Npp8u * pBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name HistogramRangeGetBufferSize
 * Companion primitives for computing the device buffer size (in bytes) required by the HistogramRange primitives.
 * @{
 */
 
#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Scratch-buffer size for nppiHistogramRange.
 * Supported data types include NPP_8U, NPP_16U, NPP_16S, NPP_32F. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels source image number of channels.
 * \param oSizeROI \ref roi_specification. 
 * \param pLevels Pointer to array of number of levels in the histogram for each eSrcDstChannels.
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return NPP_NULL_POINTER_ERROR if hpBufferSize is 0 (NULL),  \ref roi_error_codes..
 * For common parameter descriptions, see \ref CommonHistogramEvenGetBufferSizeParameters. 
 */ 
NPP_EXTERN_C
NppStatus 
nppiHistogramRangeGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                        NppiSize oSizeROI, int * pLevels, size_t * hpBufferSize /* host pointer */, 
                                        NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @} image_histogramrange */

/** 
 * \section image_proximity Image Proximity
 * @defgroup image_proximity Image Proximity
 * Primitives for computing the proximity measure between a source image and a template image.
 *
 * \section general_introduction General Introduction
 * There are basically two approaches to compute the proximity measure for template matching, 
 * Euclidean distance and the cross correlation.
 * -# Euclidean distance computes the sum of the squared distance (SSD) between the corresponding 
 * pixels of the source image and the template image. The smaller the distance is, the more 
 * similar the source image and the template image is around the pixel. The anchor of the template  
 * image is used during the computations, which always lies in the gemotric center of the image. 
 * Given a source image \f$pSrc\f$ (\f$W_s \times H_s\f$) and a template image \f$pTpl\f$ (\f$W_t \times H_t\f$), 
 * the Euclidean distance \f$D_{st}(c,r)\f$ between two images at pixel in row \f$r\f$ and column 
 * \f$c\f$ is computed as (\f$s\f$ stands for source image and \f$t\f$ for template image for short):
 * \f[D_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})]^2 \f]
 * -# Cross correlation computes the sum of the product between the corresponding pixels of the source 
 * image and the template image. The cross correlation \f$R_{st}(c,r)\f$ is calculated as:
 * \f[R_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)\cdot pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})] \f]
 * The larger the cross correlation value is, the more similar the source image and the template image is around the pixel. 
 * -# The cross correlation \f$R_{st}(c,r)\f$ is affected by the brightness of the images which may vary due to the lighting 
 * and exposure conditions. Therefore, NPP computes the cross correlation coefficient to circumvent this dependence. 
 * This is typically done at every step by subtracting the mean from every pixel value, i.e.,
 * \f[\tilde{R}_{st}(c,r)=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-Mean_t]\cdot [pSrc(j+c-\frac{H_t}{2}, i+r-\frac{W_t}{2})-Mean_s] \f]
 * 
 * NPP computes the normalized values of Euclidean distance, cross correlation and the cross correlation coefficient.
 * -# The normalized Euclidean distance \f$\sigma_{st}(c,r)\f$ is defined as:
 * \f[\sigma_{st}(c,r) = \frac{D_{st}(c,r)}{\sqrt{R_{ss}(c,r)\cdot R_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
 * -# The normalized cross correlation \f$\rho_{st}(c,r)\f$ is defined as:
 * \f[\rho_{st}(c,r) = \frac{R_{st}(c,r)}{\sqrt{R_{ss}(c,r)\cdot R_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
 * The \f$R_{ss}(c,r)\f$ and \f$R_{tt}(\frac{H_t}{2}, \frac{W_t}{2}\f$ denote the auto correlation of the source
 * image and the template image individually. They are defined as:
 * \f[R_{ss}(c,r)=\sum_{j=c-\frac{H_t}{2}}^{c+\frac{H_t}{2}}\sum_{i=r-\frac{W_t}{2}}^{r+\frac{W_t}{2}}pSrc(j, i) \f]
 * \f[R_{tt}(\frac{H_t}{2},\frac{W_t}{2})=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}pTpl(j,i) \f]
 * -# Similarly, the normalized cross correlation coefficient \f$\gamma_{st}(c,r)\f$ is calculated as:
 * \f[\gamma_{st}(c,r) = \frac{\tilde{R}_{st}(c,r)}{\sqrt{\tilde{R}_{ss}(c,r)\cdot \tilde{R}_{tt}(\frac{H_t}{2},\frac{W_t}{2})}} \f]
 * The \f$\tilde{R}_{ss}(c,r)\f$ and \f$\tilde{R}_{tt}(\frac{H_t}{2}, \frac{W_t}{2}\f$ are defined as:
 * \f[\tilde{R}_{ss}(c,r)=\sum_{j=c-\frac{H_t}{2}}^{c+\frac{H_t}{2}}\sum_{i=r-\frac{W_t}{2}}^{r+\frac{W_t}{2}}[pSrc(j, i)-Mean_s] \f]
 * \f[\tilde{R}_{tt}(\frac{H_t}{2},\frac{W_t}{2})=\sum_{j=0}^{H_t-1}\sum_{i=0}^{W_t-1}[pTpl(j,i)-Mean_t] \f] 
 * where \f$Mean_t\f$ is the template mean minus the mean of the image in the region just under the template. 
 *
 * \section category Categorizations
 * The Euclidean distance and the cross correlation are categorized into three types, full, same, and valid. 
 * -# Full mode indicates that the anchor of the template image starts from the outside of the source image, 
 * assuming the out-of-boundary pixels are zeor-padded. The size of the destination
 * image is \f$(W_s + W_t - 1) \times (H_s + H_t - 1)\f$.
 * -# Same mode means that the anchor of the template image starts from the top left pixel
 * of the source image. All the out-of-boundary pixels are also zero-padded.
 * The size of the destination image is the same as the source one, i.e., \f$W_s \times H_s\f$.
 * -# Valid mode indicates that there are no out-of-boudnary readings from the source image.
 * The anchor of the template image starts from the inside of the source image. The 
 * size of the destination image is \f$(W_s - W_t + 1) \times (H_s - H_t + 1)\f$.
 * 
 * @{
 */

/** 
 * \section sqrdistancefullnorm SqrDistanceFull_Norm 
 * @defgroup sqrdistancefullnorm SqrDistanceFull_Norm 
 * Primitives for computing the normalized Euclidean distance between two images with full mode.
 * @{
 */

/**
 * @name SqrDistanceFull_Norm
 * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction
 * with full mode (see \ref category).
 * \section CommonSqrDistanceFullNormParameters Common parameters for nppiSqrDistanceFull functions include:
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSrcRoiSize \ref roi_specification.
 * \param pTpl Pointer to the template image.
 * \param nTplStep Number of bytes between successive rows in the template image.
 * \param oTplRoiSize \ref roi_specification.
 * \param pDst \ref destination_image_pointer.
 * \param nDstStep \ref destination_image_line_step.
 * \param nScaleFactor \ref integer_result_scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image SqrDistanceFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 */ 
NPP_EXTERN_C
NppStatus 
nppiSqrDistanceFull_Norm_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                    const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                    const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                    Npp8u * pDst, int nDstStep,
                                    int nScaleFactor, NppStreamContext nppStreamCtx);

/**
 * Image SqrDistanceFull_Norm. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiSqrDistanceFull_Norm_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                             const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                             const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                             Npp32f * pDst, int nDstStep,
                             NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} SqrDistanceFull_Norm */

/**
 * \section sqrdistancesamenorm SqrDistanceSame_Norm
 * @defgroup sqrdistancesamenorm SqrDistanceSame_Norm
 * Primitives for computing the normalized Euclidean distance between two images with same mode.
 * @{
 */

/**
 * @name SqrDistanceSame_Norm
 * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category).
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image SqrDistanceSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiSqrDistanceSame_Norm_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                    const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                    const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                    Npp8u * pDst, int nDstStep,
                                    int nScaleFactor, NppStreamContext nppStreamCtx);

/**
 * Image SqrDistanceSame_Norm. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiSqrDistanceSame_Norm_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                             const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                             const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                             Npp32f * pDst, int nDstStep,
                             NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} SqrDistanceSame_Norm */

/**
 * \section sqrdistancevalidnorm SqrDistanceValid_Norm
 * @defgroup sqrdistancevalidnorm SqrDistanceValid_Norm
 * Primitives for computing the normalized Euclidean distance between two images with valid mode.
 * @{
 */

/**
 * @name SqrDistanceValid_Norm
 * The functions compute the \f$\sigma_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category).
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image SqrDistanceValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiSqrDistanceValid_Norm_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                     const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                     const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                     Npp8u * pDst, int nDstStep,
                                     int nScaleFactor, NppStreamContext nppStreamCtx);

/**
 * Image SqrDistanceValid_Norm. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiSqrDistanceValid_Norm_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                              const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                              const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                              Npp32f * pDst, int nDstStep,
                              NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} SqrDistanceValid_Norm*/

/**
 * \section crosscorrfullnorm CrossCorrFull_Norm
 * @defgroup crosscorrfullnorm CrossCorrFull_Norm
 * Primitives for computing the normalized cross correlation between two images with full mode.
 *
 * @{
 */

/**
 * @name CrossCorrFull_Norm
 * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction 
 * with full mode (see \ref category).
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrFull_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, 
 * NPP_CH_A4. 
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrFull_Norm_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                  const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                  const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                  Npp8u * pDst, int nDstStep,
                                  int nScaleFactor, NppStreamContext nppStreamCtx);

/**
 * Image CrossCorrFull_Norm with NPP_32F and NPP_64F output supported. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrFull_Norm_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                           const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                           const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                           void * pDst, int nDstStep,
                           NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrFull_Norm */

/**
 * \section crosscorrsamenorm CrossCorrSame_Norm
 * @defgroup crosscorrsamenorm CrossCorrSame_Norm
 * Primitives for computing the normalized cross correlation between two images with same mode.
 *
 * @{
 */

/**
 * @name CrossCorrSame_Norm
 * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category).
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrSame_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrSame_Norm_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                  const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                  const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                  Npp8u * pDst, int nDstStep,
                                  int nScaleFactor, NppStreamContext nppStreamCtx);

/**
 * Image CrossCorrSame_Norm with NPP_32F and NPP_64F support. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiCrossCorrSame_Norm_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                           const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                           const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                           void * pDst, int nDstStep,
                           NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrSame_Norm*/

/**
 * \section crosscorrvalidnorm CrossCorrValid_Norm
 * @defgroup crosscorrvalidnorm CrossCorrValid_Norm
 * Primitives for computing the normalized cross correlation between two images with valid mode.
 *
 * @{
 */

/**
 * @name CrossCorrValid_Norm
 * The functions compute the \f$\rho_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category).
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrValid_Norm, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_Norm_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                   const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                   const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                   Npp8u * pDst, int nDstStep,
                                   int nScaleFactor, NppStreamContext nppStreamCtx);

/**
 * Image CrossCorrValid_Norm with NPP_32F and NPP_64F support. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_Norm_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                            const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                            const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                            void * pDst, int nDstStep,
                            NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrValid_Norm*/

/**
 * \section crosscorrvalid CrossCorrValid
 * @defgroup crosscorrvalid CrossCorrValid
 * Primitives for computing the cross correlation between two images with valid mode.
 *
 * @{
 */

/**
 * @name CrossCorrValid
 * The functions compute the \f$R_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category).
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrValid with NPP_32F and NPP_64F output support (not normalized).
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                       const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                       const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                       void * pDst, int nDstStep,
                       NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrValid */

/**
 * \section crosscorrfullnormlevel CrossCorrFull_NormLevel
 * @defgroup crosscorrfullnormlevel CrossCorrFull_NormLevel
 * Primitives for computing the normalized cross correlation coefficient between two images with full mode. 
 *
 * @{
 */

/**
 * @name CrossCorrFull_NormLevel
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with full mode (see \ref category). The functions require additional scratch buffer for computations. 
 *  
 * Note: For maximum performance oSrcRoiSize.width + oTplRoiSize.width - 1 MUST be an integer multiple of 4. 
 *  
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrFull_NormLevel, scaled by \f$2^(-nScaleFactor)\f$.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrFull_NormLevel_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                       const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                       const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                       Npp8u * pDst, int nDstStep,
                                       int nScaleFactor, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Image CrossCorrFull_NormLevel with NPP_32F and NPP_64F output.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiCrossCorrFull_NormLevel_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                                const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                void * pDst, int nDstStep,
                                Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name FullNormLevelGetBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrFull_NormLevel primitives. 
 *  
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiCrossCorrFull_NormLevel.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiFullNormLevelGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                                       NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrFull_NormLevel*/

/**
 * \section crosscorrsamenormlevel CrossCorrSame_NormLevel
 * @defgroup crosscorrsamenormlevel CrossCorrSame_NormLevel
 * Primitives for computing the normalized cross correlation coefficient between two images with same mode.
 *
 * @{
 */

/**
 * @name CrossCorrSame_NormLevel
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category). The functions require additional scratch buffer for computations. 
 *  
 * Note: For maximum performance oSrcRoiSize.width MUST be an integer multiple of 4. 
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrSame_NormLevel, scaled by \f$2^(-nScaleFactor)\f$. 
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4.
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrSame_NormLevel_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                       const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                       const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                       Npp8u * pDst, int nDstStep,
                                       int nScaleFactor, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Image CrossCorrSame_NormLevel with NPP_32F and NPP_64F output.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */ 
NPP_EXTERN_C
NppStatus 
nppiCrossCorrSame_NormLevel_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                                const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                void * pDst, int nDstStep,
                                Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name SameNormLevelGetBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrSame_NormLevel primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiCrossCorrSame_NormLevel.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiSameNormLevelGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                                       NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrSame_NormLevel*/

/**
 * \section crosscorrvalidnormlevel CrossCorrValid_NormLevel
 * @defgroup crosscorrvalidnormlevel CrossCorrValid_NormLevel
 * Primitives for computing the normalized cross correlation coefficient between two images with valid mode.
 *
 * @{
 */

/**
 * @name CrossCorrValid_NormLevel
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category). The functions require additional scratch buffer for computations. 
 *  
 * Note: For maximum performance oSrcRoiSize.width - oTplRoiSize + 1 MUST be an integer multiple of 4. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrValid_NormLevel, scaled by \f$2^(-nScaleFactor)\f$.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4.
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_NormLevel_8u_Sfs_Ctx(NppiChannels eSrcDstChannels,
                                        const Npp8u * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                        const Npp8u * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                        Npp8u * pDst, int nDstStep,
                                        int nScaleFactor, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Image CrossCorrValid_NormLevel with NPP_32F and NPP_64F output.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 */ 
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_NormLevel_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                                 const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                 const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                 void * pDst, int nDstStep,
                                 Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name ValidNormLevelGetBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrValid_NormLevel primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiCrossCorrValid_NormLevel.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4, NPP_CH_A4. 
 * \param eSrcType source and template \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiValidNormLevelGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                                        NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrValid_NormLevel*/

/**
 * \section crosscorrfullnormleveladvanced CrossCorrFull_NormLevelAdvanced
 * @defgroup crosscorrfullnormleveladvanced CrossCorrFull_NormLevelAdvanced
 * Primitives for computing the normalized cross correlation coefficient between two images with full mode with large image template sizes. 
 *
 * @{
 */

/**
 * @name CrossCorrFull_NormLevelAdvanced
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with full mode (see \ref category). The functions require an additional scratch buffer and advanced scratch buffer for computations. 
 *  
 * Note: For maximum performance oSrcRoiSize.width + oTplRoiSize.width - 1 MUST be an integer multiple of 4. 
 *  
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrFull_NormLevelAdvanced with NPP_32F and NPP_64F output.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4.
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type. 
 * \param eSrcDstChannels number of channels. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrFull_NormLevelAdvanced_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                                        const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                        const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                        void * pDst, int nDstStep,
                                        Npp8u * pDeviceBuffer, Npp8u * pAdvancedScratchBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name FullNormLevelGetAdvancedScratchBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrFull_NormLevelAdvanced primitives. 
 *  
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiCrossCorrFull_NormLevelAdvanced functions. 
 *  
 * \param oSrcRoiSize \ref roi_specification.
 * \param oTplRoiSize \ref roi_specification.
 * \param nSizeofDstData sizeof(destination data type (usually Npp32f)).
 * \param nSrcChannels number of source image color channels. 
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrFull_NormLevel_GetAdvancedScratchBufferSize(NppiSize oSrcRoiSize, NppiSize oTplRoiSize, 
                                                         int nSizeofDstData, int nSrcChannels, 
                                                         size_t * hpBufferSize /* host pointer */);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrFull_NormLevelAdvanced*/

/**
 * \section crosscorrsamenormleveladvanced CrossCorrSame_NormLevelAdvanced
 * @defgroup crosscorrsamenormleveladvanced CrossCorrSame_NormLevelAdvanced
 * Primitives for computing the normalized cross correlation coefficient between two images with same mode with large image template sizes.
 * 
 * @{
 */

/**
 * @name CrossCorrSame_NormLevelAdvanced
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with same mode (see \ref category). The functions require and additional scratch buffer and advanced scratch buffer for computations. 
 *  
 * Note: For maximum performance oSrcRoiSize.width MUST be an integer multiple of 4. 
 * 
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrSame_NormLevelAdvanced with NPP_32F and NPP_64F output.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4.
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type. 
 * \param eSrcDstChannels number of channels. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrSame_NormLevelAdvanced_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                                        const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                        const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                        void * pDst, int nDstStep,
                                        Npp8u * pDeviceBuffer, Npp8u * pAdvancedScratchBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name SameNormLevelGetAdvancedScratchBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrSame_NormLevelAdvanced primitives.
 *  
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiCrossCorrSame_NormLevelAdvanced functions.
 *  
 * \param oSrcRoiSize \ref roi_specification.
 * \param oTplRoiSize \ref roi_specification.
 * \param nSizeofDstData sizeof(destination data type (usually Npp32f)).
 * \param nSrcChannels number of source image color channels. 
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrSame_NormLevel_GetAdvancedScratchBufferSize(NppiSize oSrcRoiSize, NppiSize oTplRoiSize, 
                                                         int nSizeofDstData, int nSrcChannels, 
                                                         size_t * hpBufferSize /* host pointer */);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrSame_NormLevelAdvanced */

/**
 * \section crosscorrvalidnormleveladvanced CrossCorrValid_NormLevelAdvanced
 * @defgroup crosscorrvalidnormleveladvanced CrossCorrValid_NormLevelAdvanced
 * Primitives for computing the normalized cross correlation coefficient between two images with valid mode with large template sizes.
 *
 * @{
 */

/**
 * @name CrossCorrValid_NormLevelAdvanced
 * The functions compute the \f$\gamma_{st}(c,r)\f$ in \ref general_introduction 
 * with valid mode (see \ref category). The functions require an additional scratch buffer and advanced scratch buffer for computations. 
 *  
 * Note: For maximum performance oSrcRoiSize.width - oTplRoiSize + 1 MUST be an integer multiple of 4. 
 *
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image CrossCorrValid_NormLevelAdvanced with NPP_32F and NPP_64F output.
 * For common parameter descriptions, see \ref CommonSqrDistanceFullNormParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_4.
 * \param eSrcType source and template \ref image_data_type.
 * \param eDstType destination \ref image_data_type. 
 * \param eSrcDstChannels number of channels. 
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_NormLevelAdvanced_Ctx(NppDataType eSrcType, NppDataType eDstType, NppiChannels eSrcDstChannels,
                                         const void * pSrc, int nSrcStep, NppiSize oSrcRoiSize,
                                         const void * pTpl, int nTplStep, NppiSize oTplRoiSize,
                                         void * pDst, int nDstStep,
                                         Npp8u * pDeviceBuffer, Npp8u * pAdvancedScratchBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name ValidNormLevelGetAdvancedScratchBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the CrossCorrValid_NormLevelAdvanced primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiCrossCorrValid_NormLevelAdvanced functions.
 *  
 * \param oSrcRoiSize \ref roi_specification.
 * \param oTplRoiSize \ref roi_specification.
 * \param nSizeofDstData sizeof(destination data type (usually Npp32f)).
 * \param nSrcChannels number of source image color channels. 
 * \param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiCrossCorrValid_NormLevel_GetAdvancedScratchBufferSize(NppiSize oSrcRoiSize, NppiSize oTplRoiSize, 
                                                          int nSizeofDstData, int nSrcChannels, 
                                                          size_t * hpBufferSize /* host pointer */);
#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} CrossCorrValid_NormLevelAdvanced */

/** @} image_proximity */

/** 
 * \section image_quality_index Image Quality Index
 * @defgroup image_quality_index Image Quality Index
 * Primitives for computing the image quality index of two images.
 * @{
 */

/**
 * @name QualityIndex
  * Given two images \f$M\f$ and \f$N\f$ (both \f$W \times H\f$), the
 * mathematical formula to calculate the image quality index \f$Q\f$ between them is expressed as:
 * \f[Q = \frac{4\sigma_{MN}\tilde{M}\tilde{N}}{[(\tilde{M}^2)+(\tilde{N}^2)][(\sigma_M)^2+(\sigma_N)^2]} \f]
 * where
 * \f[\tilde{M} = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}M(j,i)\f]
 * \f[\tilde{N} = \frac{1}{W\cdot H}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}N(j,i)\f]
 * \f[\sigma_{M} = \sqrt{\frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[M(j,i)-\tilde{M}]^2}\f]
 * \f[\sigma_{N} = \sqrt{\frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[N(j,i)-\tilde{N}]^2}\f]
 * \f[\sigma_{MN} = \frac{1}{W\cdot H-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}[M(j,i)-\tilde{M}][N(j,i)-\tilde{N}]\f]
 * The functions require additional scratch buffer for computations.
 * \section CommonQualityIndexParameters Common parameters for nppiQualityIndex functions include:
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcDstChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pDst Pointer to the quality index.
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes, or NPP_QUALITY_INDEX_ERROR if pixels of 
 * either image are constant numberse.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image QualityIndex.
 * For common parameter descriptions, see \ref CommonQualityIndexParameters. 
 *  
 */ 
NPP_EXTERN_C
NppStatus 
nppiQualityIndex_Ctx(NppDataType eSrcType, NppiChannels eSrcDstChannels,
                     const void * pSrc1, int nSrc1Step,
                     const void * pSrc2, int nSrc2Step,
                     NppiSize oSizeROI, Npp32f * pDst,
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name QualityIndexGetBufferHostSize 
 * Companion primitives for computing the device buffer size (in bytes) required by the QualityIndex primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size (in bytes) for nppiQualityIndex.
 * Supported data types include NPP_8U, NPP_16U, NPP_32F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3, NPP_CH_A4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels, alpha channel is ignored for NPP_CH_A4. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiQualityIndexGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                      NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @}  */
/** @} image_quality_index */

/** 
 * \section image_maximum_error MaximumError
 * @defgroup image_maximum_error MaximumError
 * Primitives for computing the maximum error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the maximum error is defined as the largest absolute difference between pixels of two images.
 * If the image is in complex format, the absolute value of the complex number is provided.
 * \section CommonMaximumErrorParameters Common parameters for nppiMaximumError functions include:
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pError Pointer to the computed error. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 * @{
 */

/** @name MaximumError
* @{
*/

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Maximum_Error.
 * For common parameter descriptions, see \ref CommonMaximumErrorParameters. 
 *  
 *  */ 
NPP_EXTERN_C
NppStatus 
nppiMaximumError_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                     const void * pSrc1, int nSrc1Step,
                     const void * pSrc2, int nSrc2Step,
                     NppiSize oSizeROI, Npp64f * pError,
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MaximumErrorGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the MaximumError primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiMaximumError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMaximumErrorGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                      NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */ 
/** @} image_maximum_error */

/** 
 * \section image_average_error AverageError
 * @defgroup image_average_error AverageError
 * Primitives for computing the average error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the average error is defined as:
 * \f[Average Error = \frac{1}{W\cdot H\cdot N}\sum_{n=0}^{N-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\left|pSrc1(j,i) - pSrc2(j,i)\right|\f]
 * where N stands for the number of channels. If the image is in complex format, the absolute value is used for computation.
 * @{
 */

/** @name AverageError
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image Average_Error.
 * For common parameter descriptions, see \ref CommonMaximumErrorParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4.
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pError Pointer to the computed error. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
*/
NPP_EXTERN_C
NppStatus 
nppiAverageError_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                     const void * pSrc1, int nSrc1Step,
                     const void * pSrc2, int nSrc2Step,
                     NppiSize oSizeROI, Npp64f * pError,
                     Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name AverageErrorGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiAverageError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiAverageErrorGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                      NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */
/** @} image_average_error */

/** 
 * \section image_maximum_relative_error MaximumRelativeError
 * @defgroup image_maximum_relative_error MaximumRelativeError
 * Primitives for computing the maximum relative error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the maximum relative error is defined as:
 * \f[MaximumRelativeError = max{\frac{\left|pSrc1(j,i) - pSrc2(j,i)\right|}{max(\left|pSrc1(j,i)\right|, \left|pSrc2(j,i)\right|)}}\f]
 * If the image is in complex format, the absolute value is used for computation.
 * For multiple channles, the maximum relative error of all the channles is returned.
 * @{
 */

/** @name MaximumRelativeError
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image MaximumRelative_Error.
 * For common parameter descriptions, see \ref CommonMaximumErrorParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4.
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pError Pointer to the computed error. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
*/
NPP_EXTERN_C
NppStatus 
nppiMaximumRelativeError_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                             const void * pSrc1, int nSrc1Step,
                             const void * pSrc2, int nSrc2Step,
                             NppiSize oSizeROI, Npp64f * pError,
                             Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MaximumRelativeErrorGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiMaximumRelativeError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMaximumRelativeErrorGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                              NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @} image_maximum_relative_error */

/** 
 * \section image_average_relative_error AverageRelativeError
 * @defgroup image_average_relative_error AverageRelativeError
 * Primitives for computing the average relative error between two images.
 * Given two images \f$pSrc1\f$ and \f$pSrc2\f$ both with width \f$W\f$ and height \f$H\f$, 
 * the maximum relative error is defined as:
 * \f[AverageRelativeError = \frac{1}{W\cdot H\cdot N}\sum_{n=0}^{N-1}\sum_{j=0}^{H-1}\sum_{i=0}^{W-1}\frac{\left|pSrc1(j,i) - pSrc2(j,i)\right|}{max(\left|pSrc1(j,i)\right|, \left|pSrc2(j,i)\right|)}\f]
 * where N is the number of channels.
 * If the image is in complex format, the absolute value is used for computation.
 * @{
 */

/** @name AverageRelativeError
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * Image MaximumRelative_Error.
 * For common parameter descriptions, see \ref CommonMaximumErrorParameters. 
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4.
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pError Pointer to the computed error. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
*/
NPP_EXTERN_C
NppStatus 
nppiAverageRelativeError_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                             const void * pSrc1, int nSrc1Step,
                             const void * pSrc2, int nSrc2Step,
                             NppiSize oSizeROI, Npp64f * pError,
                             Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name AverageRelativeErrorGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the NormDiff_Inf primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiAverageRelativeError.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_32F, NPP_64F. 
 * Supported channel counts include NPP_CH_1, NPP_CH_2, NPP_CH_3, NPP_CH_4. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiAverageRelativeErrorGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                              NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @} image_average_relative_error */

/** 
 * \section image_quality_assessment IQA
 * @defgroup image_quality_assessment IQA
 * Primitives for computing the image quality between two images, such as MSE, PSNR, SSIM, and MS-SSIM.
 * @{
 */

/** @name MSE
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned image MSE.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMSE Device memory pointer to the computed MSE of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
*/
NPP_EXTERN_C
NppStatus 
nppiMSE_8u_C1R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                   const Npp8u * pSrc2, int nSrc2Step, 
                   NppiSize oSizeROI, Npp32f * pMSE, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Three-channel 8-bit unsigned image MSE.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMSE Device memory pointer to the computed MSE of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
*/

NPP_EXTERN_C
NppStatus 
nppiMSE_8u_C3R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                   const Npp8u * pSrc2, int nSrc2Step, 
                   NppiSize oSizeROI, Npp32f * pMSE, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MSE */

/** @name PSNR
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned image PSNR.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pPSNR Device memory pointer to the computed PSNR of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiPSNR_8u_C1R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                    const Npp8u * pSrc2, int nSrc2Step, 
                    NppiSize oSizeROI, Npp32f * pPSNR, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Three-channel 8-bit unsigned image PSNR.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pPSNR Device memory pointer to the computed PSNR of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiPSNR_8u_C3R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                    const Npp8u * pSrc2, int nSrc2Step, 
                    NppiSize oSizeROI, Npp32f * pPSNR, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name MSEAndPSNRGetBufferHostSize
 * Companion primitives for computing the device buffer size (in bytes) required by the MSE and PSNR primitives.
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiMSE, and nppiPSNR.
 * Supported data types include NPP_8U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMSEAndPSNRGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                    NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} */

/** @name SSIM
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned image SSIM.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pSSIM Device memory pointer to the computed SSIM of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiSSIM_8u_C1R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                    const Npp8u * pSrc2, int nSrc2Step, 
                    NppiSize oSizeROI, Npp32f * pSSIM, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Three-channel 8-bit unsigned image SSIM*.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pSSIM Device memory pointer to the computed SSIM of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiSSIM_8u_C3R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                    const Npp8u * pSrc2, int nSrc2Step, 
                    NppiSize oSizeROI, Npp32f * pSSIM, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} SSIM */

/** @name MSSSIM
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned image MS-SSIM*.
 * This function will be deprecated in a future release use the nppiWMSSSIM functions instead.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMSSSIM Device memory pointer to the computed MS-SSIM of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiMSSSIM_8u_C1R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                      const Npp8u * pSrc2, int nSrc2Step, 
                      NppiSize oSizeROI, Npp32f * pMSSSIM, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MSSSIM */

/** @name WMSSSIM
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned image MS-SSIM*.
 * This function uses the algorithm described in the paper by Wang et. al.
 * Wang, Z., Simoncelli, E.P., Bovik, A.C. Multiscale Structural Similarity for Image Quality Assessment. In: The
 * Thirty-Seventh Asilomar Conference on Signals, Systems & Computers, 2003, 13981402. Pacific Grove, CA, USA: IEEE,
 * 2003. https://doi.org/10.1109/ACSSC.2003.1292216.
 * NOTE: this API call can only process oSizeROI dimensions 16px by 16px and above. Any oSizeROI dimensions less than 16px by 16px
 * will result in undefined behaviour.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMSSSIM Device memory pointer to the computed MS-SSIM of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIM_8u_C1R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                       const Npp8u * pSrc2, int nSrc2Step, 
                       NppiSize oSizeROI, Npp32f * pMSSSIM, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

/**
 * Three-channel 8-bit unsigned image MS-SSIM*.
 * This function uses the algorithm described in the paper by Wang et. al.
 * Wang, Z., Simoncelli, E.P., Bovik, A.C. Multiscale Structural Similarity for Image Quality Assessment. In: The
 * Thirty-Seventh Asilomar Conference on Signals, Systems & Computers, 2003, 13981402. Pacific Grove, CA, USA: IEEE,
 * 2003. https://doi.org/10.1109/ACSSC.2003.1292216.
 * NOTE: this API call can only process oSizeROI dimensions 16px by 16px and above. Any oSizeROI dimensions less than 16px by 16px
 * will result in undefined behaviour.
 *
 * \param pSrc1 \ref source_image_pointer.
 * \param nSrc1Step \ref source_image_line_step.
 * \param pSrc2 \ref source_image_pointer.
 * \param nSrc2Step \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param pMSSSIM Device memory pointer to the computed MS-SSIM of two images. 
 * \param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref image_data_error_codes, \ref roi_error_codes.
*/
NPP_EXTERN_C
NppStatus 
nppiWMSSSIM_8u_C3R_Ctx(const Npp8u * pSrc1, int nSrc1Step, 
                       const Npp8u * pSrc2, int nSrc2Step, 
                       NppiSize oSizeROI, Npp32f * pMSSSIM, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} WMSSSIM */

/** @name MSEGetBufferHostSize
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for nppiSSIM_8u_C1R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiSSIMGetBufferHostSize_8u_C1R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * Buffer size for nppiSSIM_8u_C3R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiSSIMGetBufferHostSize_8u_C3R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * Buffer size for nppiMSSSIM_8u_C1R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiMSSSIMGetBufferHostSize_8u_C1R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);


/** 
 * Buffer size for nppiWMSSSIM_8u_C1R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMGetBufferHostSize_8u_C1R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * Buffer size for nppiWMSSSIM_8u_C3R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMGetBufferHostSize_8u_C3R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MSEGetBufferHostSize */

/** @} image_quality_assessment */

/** 
 * \section batch_image_quality_assessment IQABatch
 * @defgroup batch_image_quality_assessment IQABatch
 * Primitives for computing the image quality for a batch of image pairs, such as MSE, PSNR, SSIM, MS-SSIM, and WMMS-SSIM with a single \ref
 * roi_specification for all pairs of input images
 * @{
 */

/** @name MSEBatch
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned MSE for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pMSE Device memory pointer to output array of the computed MSE for nBatchSize * sizeof(Npp32f) * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiMSEBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMSEBatch_8u_C1R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                        const NppiImageDescriptor * pSrc2BatchList, 
                        int nBatchSize, NppiSize oSizeROI, Npp32f * pMSE, 
                        NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned MSE for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pMSE Device memory pointer to output array of the computed MSE for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiMSEBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMSEBatch_8u_C3R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                        const NppiImageDescriptor * pSrc2BatchList, 
                        int nBatchSize, NppiSize oSizeROI, Npp32f * pMSE, 
                        NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MSE */

/** @name PSNRBatch
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned PSNR for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pPSNR Device memory pointer to output array of the computed PSNR for nBatchSize * sizeof(Npp32f) * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiPSNRBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiPSNRBatch_8u_C1R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                         const NppiImageDescriptor * pSrc2BatchList, 
                         int nBatchSize, NppiSize oSizeROI, Npp32f * pPSNR, 
                         NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned PSNR for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pPSNR Device memory pointer to output array of the computed PSNR for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiPSNRBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiPSNRBatch_8u_C3R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                         const NppiImageDescriptor * pSrc2BatchList, 
                         int nBatchSize, NppiSize oSizeROI, Npp32f * pPSNR, 
                         NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} PSNR */

/** @name MSEAndPSNRBatchGetBufferHostSize
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for a single image pair in the batch of image pairs for nppiMSEBatch and nppiPSNRBatch.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 * Supported data types include NPP_8U. 
 * Supported channel counts include NPP_CH_1, NPP_CH_3. 
 * \param eSrcType source \ref image_data_type.
 * \param eSrcChannels \ref image_channels. 
 * \param oSizeROI \ref roi_specification. 
 * \param hpBufferSize Host memory pointer to required byte count value. 
 * \param nppStreamCtx \ref application_managed_stream_context. 
 */
NPP_EXTERN_C
NppStatus 
nppiMSEAndPSNRBatchGetBufferHostSize_Ctx(NppDataType eSrcType, NppiChannels eSrcChannels,
                                         NppiSize oSizeROI, size_t * hpBufferSize /* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MSEAndPSNRBatchGetBufferHostSize */

/** @name SSIMBatch
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned SSIM for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pSSIM Device memory pointer to output array of the computed SSIM for nBatchSize * sizeof(Npp32f) * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiSSIMBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSSIMBatch_8u_C1R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                         const NppiImageDescriptor * pSrc2BatchList, 
                         int nBatchSize, NppiSize oSizeROI, Npp32f * pSSIM, 
                         NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned SSIM for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pSSIM Device memory pointer to output array of the computed SSIM for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiSSIMBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiSSIMBatch_8u_C3R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                         const NppiImageDescriptor * pSrc2BatchList, 
                         int nBatchSize, NppiSize oSizeROI, Npp32f * pSSIM, 
                         NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} SSIM */

/** @name WMSSSIMBatch
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned WMSSSIM for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 * NOTE: this API call can only process oSizeROI dimensions 16 pixels by 16 pixels and above. Any oSizeROI dimensions less than 16 pixels by 16 pixels
 * will result in undefined behaviour.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pWMSSSIM Device memory pointer to output array of the computed WMSSSIM for nBatchSize * sizeof(Npp32f) * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 *  memory buffer pointers and size as returned by at least one nppiWMSSSIMBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMBatch_8u_C1R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                            const NppiImageDescriptor * pSrc2BatchList, 
                            int nBatchSize, NppiSize oSizeROI, Npp32f * pWMSSSIM, 
                            NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned WMSSSIM for a batch of image pairs for a single ROI.
 * Provided oSizeROI will be used for all images passed in pSrc1BatchList and pSrc2BatchList
 * arguments. API user must ensure that provided ROI (oSizeROI) does not go beyond the borders of any of provided images.
 * NOTE: this API call can only process ROI dimensions 16 pixels by 16 pixels and above. Any ROI dimensions less than 16 pixels by 16 pixels
 * will result in undefined behaviour.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors, per image oSize must be initialized.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oSizeROI \ref roi_specification ROI width and height of ALL images in the batch, MUST match the ROI used when the label markers UF image was generated.
 * \param pWMSSSIM Device memory pointer to output array of the computed WMSSSIM for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiWMSSSIMBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMBatch_8u_C3R_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                            const NppiImageDescriptor * pSrc2BatchList, 
                            int nBatchSize, NppiSize oSizeROI, Npp32f * pWMSSSIM, 
                            NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} WMSSSIMBatch */

/** @name SSIMBatchGetBufferHostSize
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Buffer size for a single image pair in the batch of image pairs for nppiSSIMBatch_8u_C1R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiSSIMBatchGetBufferHostSize_8u_C1R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * Buffer size for a single image pair in the batch of image pairs for nppiSSIMBatch_8u_C3R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiSSIMBatchGetBufferHostSize_8u_C3R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

/** 
 * Buffer size for a single image pair in the batch of image pairs for nppiWMSSSIMBatch_8u_C1R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMBatchGetBufferHostSize_8u_C1R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);


/** 
 * Buffer size for a single image pair in the batch of image pairs for nppiWMSSSIMBatch_8u_C3R.
 * For common parameter descriptions, see \ref CommonGetBufferHostSizeParameters.
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMBatchGetBufferHostSize_8u_C3R_Ctx(NppiSize oSizeROI, size_t * hpBufferSize/* host pointer */, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} SSIMBatchGetBufferHostSize */

/** @} batch_image_quality_assessment */


/** 
 * \section batch_image_quality_assessment_advanced IQABatchAdvanced
 * @defgroup batch_image_quality_assessment_advanced IQABatchAdvanced
 * Primitives for computing the image quality for a batch of image pairs, such as MSE, PSNR, SSIM, MS-SSIM, and WMMS-SSIM with per-image \ref
 * roi_specification
 * @{
 */

/** @name MSEBatchAdvanced
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned MSE for a batch of image pairs with per-image ROI
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oMaxSizeROI \ref roi_specification maximum ROI width and height of ALL images in the batch.
 * \param pMSE Device memory pointer to output array of the computed MSE for nBatchSize * sizeof(Npp32f * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiMSEBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMSEBatch_8u_C1R_Advanced_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                                 const NppiImageDescriptor * pSrc2BatchList, 
                                 int nBatchSize, NppiSize oMaxSizeROI, Npp32f * pMSE, 
                                 NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned MSE for a batch of image pairs with per-image ROI
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oMaxSizeROI \ref roi_specification maximum ROI width and height of ALL images in the batch.
 * \param pMSE Device memory pointer to output array of the computed MSE for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiMSEBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiMSEBatch_8u_C3R_Advanced_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                                 const NppiImageDescriptor * pSrc2BatchList, 
                                 int nBatchSize, NppiSize oMaxSizeROI, Npp32f * pMSE, 
                                 NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} MSEBatchAdvanced */

/** @name PSNRBatchAdvanced
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned PSNR for a batch of image pairs with per-image ROI
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oMaxSizeROI \ref roi_specification maximum ROI width and height of ALL images in the batch.
 * \param pPSNR Device memory pointer to output array of the computed PSNR for nBatchSize * sizeof(Npp32f) * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 *  memory buffer pointers and size as returned by at least one nppiPSNRBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiPSNRBatch_8u_C1R_Advanced_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                                  const NppiImageDescriptor * pSrc2BatchList, 
                                  int nBatchSize, NppiSize oMaxSizeROI, Npp32f * pPSNR, 
                                  NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned PSNR for a batch of image pairs with per-image ROI
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oMaxSizeROI \ref roi_specification maximum ROI width and height of ALL images in the batch.
 * \param pPSNR Device memory pointer to output array of the computed PSNR for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiPSNRBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiPSNRBatch_8u_C3R_Advanced_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                                  const NppiImageDescriptor * pSrc2BatchList, 
                                  int nBatchSize, NppiSize oMaxSizeROI, Npp32f * pPSNR, 
                                  NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);
#ifdef NPP_PLUS
}
#endif
/** @} PSNRBatchAdvanced */

/** @name WMSSSIMBatchAdvanced
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * One channel 8-bit unsigned WMSSSIM for a batch of image pairs with per-image ROI
 * NOTE: It is the user's responsibility to make sure the dimensions of per-image ROIs are 16 pixels by 16 pixels and above. Any per-image ROI dimensions less than 16 pixels by 16 pixels will result in undefined behaviour.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oMaxSizeROI \ref roi_specification maximum ROI width and height of ALL images in the batch.
 * \param pWMSSSIM Device memory pointer to output array of the computed WMSSSIM for nBatchSize * sizeof(Npp32f) * 1 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 *  memory buffer pointers and size as returned by at least one nppiWMSSSIMBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMBatch_8u_C1R_Advanced_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                                     const NppiImageDescriptor * pSrc2BatchList, 
                                     int nBatchSize, NppiSize oMaxSizeROI, Npp32f * pWMSSSIM, 
                                     NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

/**
 * 3 channel 8-bit unsigned WMSSSIM for a batch of image pairs with per-image ROI
 * NOTE: It is the user's responsibility to make sure the dimensions of per-image ROIs are 16 pixels by 16 pixels and above. Any per-image ROI dimensions less than 16 pixels by 16 pixels will result in undefined behaviour.
 *
 * \param pSrc1BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param pSrc2BatchList \ref source_batch_images_pointer device memory pointer to the list of device memory image descriptors.
 * \param nBatchSize Number of \ref NppiImageDescriptor, NppiBufferDescriptor, and new max number structures/values processed in this call (must be > 1).
 * \param oMaxSizeROI \ref roi_specification maximum ROI width and height of ALL images in the batch.
 * \param pWMSSSIM Device memory pointer to output array of the computed WMSSSIM for nBatchSize * sizeof(Npp32f) * 3 image pairs. 
 * \param pDeviceBufferList Device memory pointer to the list of NppiBufferDescriptor buffer descriptors specifying per image device
 * memory buffer pointers and size as returned by at least one nppiWMSSSIMBatchGetBufferHostSize call.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes
 */
NPP_EXTERN_C
NppStatus 
nppiWMSSSIMBatch_8u_C3R_Advanced_Ctx(const NppiImageDescriptor * pSrc1BatchList, 
                                     const NppiImageDescriptor * pSrc2BatchList, 
                                     int nBatchSize, NppiSize oMaxSizeROI, Npp32f * pWMSSSIM, 
                                     NppiBufferDescriptor * pDeviceBufferList, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} WMSSSIMBatchAdvanced */

/** @} batch_image_quality_assessment_advanced */

/** @name Profiles
 * @{
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
 /**
  * One-channel 32-bit Circular radial profile based upon a specified center point.
  *
  * \param pSrc \ref source_image_pointer.
  * \param nSrcStep \ref source_image_line_step.
  * \param oSizeROI \ref roi_specification.
  * \param oCenter center of circle that radial profile is computed from.  Center does not have to be on the image.
  * \param pProfileData pointer to output NppiProfileData array containing count, mean, and std_dev as a function of radius.
  * \param nProfileSamples number of radii to be sampled: from 0 through nProfileSamples-1.
  * \param nppStreamCtx \ref application_managed_stream_context.
  * \return \ref image_data_error_codes, \ref roi_error_codes.
 */
NPP_EXTERN_C
NppStatus nppiCircularRadialProfile_32f_C1R_Ctx(const Npp32f * pSrc, int nSrcStep, NppiSize oSizeROI, NppiPoint32f oCenter,
    NppiProfileData *pProfileData, Npp16u nProfileSamples, NppStreamContext nppStreamCtx);

/**
 * One-channel 32-bit Elliptical radial profile based upon a specified center point, angle, and astigmatism.
 *
 * \param pSrc \ref source_image_pointer.
 * \param nSrcStep \ref source_image_line_step.
 * \param oSizeROI \ref roi_specification.
 * \param oCenter center of circle that radial profile is computed from.  Center does not have to be on the image.
 * \param nAstigmatismRatio ratio of orientation axis to orthogonal axis.  (i.e. 1.5 means stretched 1.5x)
 * \param nOrientationRadians orientation of the astigmatism, clockwise in radians with 0.0 being vertical.
 * \param pProfileData pointer to output NppiProfileData array containing count, mean, and std_dev as a function of radius.
 * \param nProfileSamples number of radii to be sampled: from 0 through nProfileSamples-1.
 * \param nppStreamCtx \ref application_managed_stream_context.
 * \return \ref image_data_error_codes, \ref roi_error_codes.
*/
NPP_EXTERN_C
NppStatus nppiEllipticalRadialProfile_32f_C1R_Ctx(const Npp32f * pSrc, int nSrcStep, NppiSize oSizeROI, NppiPoint32f oCenter,
    Npp32f nAstigmatismRatio, Npp32f nOrientationRadians, NppiProfileData *pProfileData, Npp16u nProfileSamples, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} Profiles */

/** @} */

/** @} image_statistics_functions */

#endif /* NV_NPPI_PLUS_STATISTICS_FUNCTIONS_H */

